from steamship import File
from tests.demo_apps.plugins.importers.plugin_file_importer import TEST_DOC

from .. import APPS_PATH

__copyright__ = "Steamship"
__license__ = "MIT"

from ..utils.client import get_steamship_client
from ..utils.deployables import deploy_plugin


def test_e2e_importer():
    client = get_steamship_client()
    file_importer_path = APPS_PATH / "plugins" / "importers" / "plugin_file_importer.py"

    with deploy_plugin(client, file_importer_path, "fileImporter") as (
        plugin,
        version,
        instance,
    ):
        # The test FileImporter should always return a string file with contents TEST_DOC
        file = File.create(
            client=client, content="This is a test.", plugin_instance=instance.handle
        ).data

        # Now fetch the data from Steamship and assert that it is the SAME as the data the FileImporter creates
        data = file.raw().data
        assert data.decode("utf-8") == TEST_DOC

        file.delete()



import json

from steamship import Tag
from steamship.app import App, Response, create_handler, post
from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginRequest
from steamship.plugin.tagger import Tagger


class TestParserPlugin(Tagger, App):
    # For testing; mirrors TestConfigurableTagger in Swift

    def run(
        self, request: PluginRequest[BlockAndTagPluginInput]
    ) -> Response[BlockAndTagPluginOutput]:
        tag_kind = self.config["tagKind"]  # TODO (enias): Review config loading
        tag_name = self.config["tagName"]
        tag_value = json.dumps(
            {
                "numberValue": self.config["numberValue"],
                "booleanValue": self.config["booleanValue"],
            }
        )

        if request.data is not None:
            file = request.data.file
            tag = Tag.CreateRequest(kind=tag_kind, name=tag_name, value=tag_value)
            if file.tags:
                file.tags.append(tag)
            else:
                file.tags = [tag]
            return Response(data=BlockAndTagPluginOutput(file))

    @post("tag")
    def parse(self, **kwargs) -> dict:
        parse_request = Tagger.parse_request(request=kwargs)
        return self.run(parse_request)


handler = create_handler(TestParserPlugin)



"""CSV Blockifier - Steamship Plugin."""

from typing import Any, Dict

from steamship.app import App, Response, create_handler, post
from steamship.plugin.blockifier import Blockifier
from tests.demo_apps.plugins.blockifiers.csv_blockifier import CsvBlockifier


class TsvBlockifier(CsvBlockifier, Blockifier, App):
    """Converts TSV into Tagged Steamship Blocks.

    Implementation is only here to demonstrate how plugins can be built through inheritance.
    """

    def __init__(self, client=None, config: Dict[str, Any] = None):
        super().__init__(client, config)
        self.config.delimiter = "\t"

    @post("blockify")  # TODO (enias): Move the blockify handle one inheritance level up
    def blockify(self, **kwargs) -> Response:
        """App endpoint for our plugin.

        The `run` method above implements the Plugin interface for a Converter.
        This `convert` method exposes it over an HTTP endpoint as a Steamship App.

        When developing your own plugin, you can almost always leave the below code unchanged.
        """
        blockify_request = Blockifier.parse_request(request=kwargs)
        return self.run(blockify_request)


handler = create_handler(TsvBlockifier)



import contextlib
import io
import os
import time
import zipfile
from pathlib import Path
from typing import Any, Dict

from steamship import App, AppInstance, AppVersion, Steamship
from steamship.data.plugin import Plugin
from steamship.data.plugin_instance import PluginInstance
from steamship.data.plugin_version import PluginVersion
from steamship.data.user import User
from tests import SRC_PATH, VENV_PATH


def zip_deployable(file_path: Path) -> bytes:
    """Prepare and zip a Steamship plugin."""

    # TODO: This is very dependent on the setup of the local machine.
    # Might be good to find a more machine-invariant solution here.
    # The goal is to copy in all the dependencies of the lambda package.
    # Which are: steamship (current repo), setuptools_scm, requests
    dependencies_path = VENV_PATH / "lib" / "python3.9" / "site-packages"

    package_paths = [
        SRC_PATH / "steamship",
        SRC_PATH
        / ".."
        / "tests",  # This is included to test plugin development using inheritance
        dependencies_path / "setuptools_scm",
        dependencies_path / "requests",
        dependencies_path / "charset_normalizer",
        dependencies_path / "certifi",
        dependencies_path / "urllib3",
        dependencies_path / "idna",
        dependencies_path / "pydantic",
    ]

    zip_buffer = io.BytesIO()
    with zipfile.ZipFile(
        file=zip_buffer, mode="a", compression=zipfile.ZIP_DEFLATED, allowZip64=False
    ) as zip_file:
        zip_file.write(file_path, "api.py")

        for package_path in package_paths:
            for root, _, files in os.walk(package_path):
                for file in files:
                    pypi_file = Path(root) / file
                    zip_file.write(
                        pypi_file, pypi_file.relative_to(package_path.parent)
                    )

    return zip_buffer.getvalue()


@contextlib.contextmanager
def deploy_app(
    client: Steamship,
    py_path: Path,
    version_config_template: Dict[str, Any] = None,
    instance_config: Dict[str, Any] = None,
):
    app = App.create(client)
    assert app.error is None
    assert app.data is not None
    app = app.data

    zip_bytes = zip_deployable(py_path)
    version = AppVersion.create(
        client,
        app_id=app.id,
        filebytes=zip_bytes,
        config_template=version_config_template,
    )

    version = _wait_for_version(version)
    instance = AppInstance.create(
        client, app_id=app.id, app_version_id=version.id, config=instance_config
    )
    instance = _wait_for_instance(instance)

    assert instance.app_id == app.id
    assert instance.app_version_id == version.id

    _check_user(client, instance)

    yield app, version, instance

    _delete_deployable(instance, version, app)


@contextlib.contextmanager
def deploy_plugin(
    client: Steamship,
    py_path: Path,
    plugin_type: str,
    version_config_template: Dict[str, Any] = None,
    instance_config: Dict[str, Any] = None,
    training_platform: str = None,
):
    plugin = Plugin.create(
        client,
        training_platform=training_platform,
        description="test",
        type_=plugin_type,
        transport="jsonOverHttp",
        is_public=False,
    )
    assert plugin.error is None
    assert plugin.data is not None
    plugin = plugin.data

    zip_bytes = zip_deployable(py_path)
    version = PluginVersion.create(
        client,
        "test-version",
        plugin_id=plugin.id,
        filebytes=zip_bytes,
        config_template=version_config_template,  # TODO: What is this?
    )
    # TODO: This is due to having to wait for the lambda to finish deploying.
    # TODO: We should update the task system to allow its .wait() to depend on this.
    version = _wait_for_version(version)

    instance = PluginInstance.create(
        client,
        plugin_id=plugin.id,
        plugin_version_id=version.id,
        config=instance_config,
    )
    instance = _wait_for_instance(instance)

    assert instance.plugin_id == plugin.id
    assert instance.plugin_version_id == version.id

    _check_user(client, instance)

    yield plugin, version, instance

    _delete_deployable(instance, version, plugin)


def _check_user(client, instance):
    user = User.current(client).data
    assert instance.user_id == user.id


def _delete_deployable(instance, version, deployable):
    res = instance.delete()
    assert res.error is None
    res = version.delete()
    assert res.error is None
    res = deployable.delete()
    assert res.error is None


def _wait_for_instance(instance):
    instance.wait()
    assert instance.error is None
    assert instance.data is not None
    instance = instance.data
    return instance


def _wait_for_version(version):
    time.sleep(15)
    version.wait()
    assert version.error is None
    assert version.data is not None
    version = version.data
    return version



from steamship import App

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_app_create():
    client = get_steamship_client()

    app = App.create(client)
    assert app.error is None
    assert app.data is not None

    _ = app.data.delete()
    assert app.error is None



import base64
from dataclasses import dataclass
from typing import Any, Optional

from steamship.base import Client
from steamship.base.mime_types import TEXT_MIME_TYPES


def is_base64(sb):
    # noinspection PyBroadException
    try:
        if isinstance(sb, str):
            # If there's Any unicode here, an exception will be thrown and the function will return false
            sb_bytes = bytes(sb, "ascii")
        elif isinstance(sb, bytes):
            sb_bytes = sb
        else:
            raise ValueError("Argument must be string or bytes")
        return base64.b64encode(base64.b64decode(sb_bytes)) == sb_bytes
    except Exception:
        return False


@dataclass
class RawDataPluginInput:
    pluginInstance: str = None
    data: Any = None
    defaultMimeType: str = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "Optional[RawDataPluginInput]":
        if d is None:
            return None
        data = d.get("data", None)

        # TODO: We need to do a pass on proper encoding across the engine and clients.
        # if data is not None and d.get('isBase64', False):
        #     data_bytes = base64.b64decode(data)
        #     if d.get('defaultMimeType', None) in TEXT_MIME_TYPES:
        #         data = data_bytes.decode('utf-8')
        #     else:
        #         data = data_bytes
        if data is not None and is_base64(data):
            data_bytes = base64.b64decode(data)
            if d.get("defaultMimeType", None) in TEXT_MIME_TYPES:
                data = data_bytes.decode("utf-8")
            else:
                data = data_bytes

        return RawDataPluginInput(
            pluginInstance=d.get("pluginInstance", None),
            data=data,
            defaultMimeType=d.get("defaultMimeType", None),
        )



from steamship.data.user import User

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_connect():
    """Test basic connection"""
    client = get_steamship_client()
    assert client.config is not None
    assert client.config.profile == "test"
    assert client.config.api_key is not None


def test_user():
    client = get_steamship_client()
    user = User.current(client).data
    assert user.id is not None
    assert user.handle is not None



from typing import Any

from steamship import File
from steamship.app import App, Response, create_handler, post
from steamship.client.operations.corpus_importer import (
    CorpusImportRequest,
    CorpusImportResponse,
)
from steamship.plugin.corpus_importer import CorpusImporter
from steamship.plugin.service import PluginRequest


class TestCorpusImporterPlugin(CorpusImporter, App):
    def run(
        self, request: PluginRequest[CorpusImportRequest]
    ) -> Response[CorpusImportResponse]:
        return Response(
            data=CorpusImportResponse(
                file_import_requests=[
                    File.CreateRequest(
                        type="fileImporter",
                        pluginInstance=request.data.fileImporterPluginInstance
                        # This is a test importer built into the Steamship Engine
                    ),
                    File.CreateRequest(
                        type="fileImporter",
                        pluginInstance=request.data.fileImporterPluginInstance,
                    ),
                ]
            )
        )

    @post("import")
    def do_import(self, **kwargs) -> Any:
        request = CorpusImporter.parse_request(request=kwargs)
        return self.run(request)


handler = create_handler(TestCorpusImporterPlugin)



"""Collection of unit tests and integration tests for the steamship client."""
from pathlib import Path

ROOT_PATH = Path(__file__).parent.parent
VENV_PATH = ROOT_PATH / ".venv"
SRC_PATH = ROOT_PATH / "src"
TEST_ASSETS_PATH = ROOT_PATH / "test_assets"
APPS_PATH = Path(__file__).parent / "demo_apps"



import time

from steamship.base.tasks import *

T = TypeVar("T")  # Declare type variable


@dataclass
class Response(IResponse, Generic[T]):
    expect: Type[T] = None
    task: Task = None
    data: T = None
    error: SteamshipError = None
    client: Any = None

    def update(self, response: "Response[T]"):
        if self.task is not None and response.task is not None:
            self.task.update(response.task)
        if response.data is not None:
            self.data = response.data
        self.error = response.error

    def wait(self, max_timeout_s: float = 60, retry_delay_s: float = 1):
        """Polls and blocks until the task has succeeded or failed (or timeout reached)."""
        start = time.time()
        if self.task is None or self.task.state == TaskState.failed:
            return

        self.check()
        if self.task is not None:
            if (
                self.task.state == TaskState.succeeded
                or self.task.state == TaskState.failed
            ):
                return
        else:
            return
        time.sleep(retry_delay_s)

        while time.time() - start < max_timeout_s:
            time.sleep(retry_delay_s)
            self.check()
            if self.task is not None:
                if (
                    self.task.state == TaskState.succeeded
                    or self.task.state == TaskState.failed
                ):
                    return
            else:
                return

    def check(self):
        if self.task is None:
            return
        req = TaskStatusRequest(self.task.task_id)
        resp = self.client.post("task/status", payload=req, expect=self.expect)
        self.update(resp)

    def add_comment(
        self,
        external_id: str = None,
        external_type: str = None,
        external_group: str = None,
        metadata: Any = None,
    ) -> "Response[TaskComment]":
        if self.task is not None:
            return self.task.add_comment(
                external_id=external_id,
                external_type=external_type,
                external_group=external_group,
                metadata=metadata,
            )

    def list_comments(self) -> "Response[TaskCommentList]":
        if self.task is not None:
            return self.task.list_comments()

    def delete_comment(self, comment: "TaskComment" = None) -> "Response[TaskComment]":
        if self.task is not None:
            return self.task.delete_comment(comment=comment)



from typing import Any

from steamship import MimeTypes
from steamship.app import App, Response, create_handler, post
from steamship.extension.file import File
from steamship.plugin.file_importer import FileImporter
from steamship.plugin.inputs.file_import_plugin_input import FileImportPluginInput
from steamship.plugin.outputs.raw_data_plugin_output import RawDataPluginOutput
from steamship.plugin.service import PluginRequest

# Note: this aligns with the same document in the internal Engine test.
HANDLE = "test-importer-plugin-v1"
TEST_H1 = "A Poem"
TEST_S1 = "Roses are red."
TEST_S2 = "Violets are blue."
TEST_S3 = "Sugar is sweet, and I love you."
TEST_DOC = f"# {TEST_H1}\n\n{TEST_S1} {TEST_S2}\n\n{TEST_S3}\n"


class TestFileImporterPlugin(FileImporter, App):
    def run(
        self, request: PluginRequest[FileImportPluginInput]
    ) -> Response[RawDataPluginOutput]:
        return Response(
            data=RawDataPluginOutput(
                string=TEST_DOC,
                mime_type=MimeTypes.MKD
            )
        )

    @post("import")
    def do_import(self, **kwargs) -> Any:
        # TODO (enias): Move this code up one abstraction level
        import_request = FileImporter.parse_request(request=kwargs)
        return self.run(import_request)


handler = create_handler(TestFileImporterPlugin)



"""

Please see https://docs.steamship.com/ for information about building a Steamship App

"""

import logging
from collections import defaultdict
from functools import wraps
from http import HTTPStatus
from typing import Any, Dict, Optional

from steamship.app.request import Request, Verb
from steamship.app.response import Response
from steamship.base import Client


def make_registering_decorator(decorator):  # TODO (Enias): Review
    """
    Returns a copy of foreignDecorator, which is identical in every
    way(*), except also appends a .decorator property to the callable it
    spits out.

    (*)We can be somewhat "hygienic", but newDecorator still isn't signature-preserving,
    i.e. you will not be able to get a runtime list of parameters.
    For that, you need hackish libraries...but in this case, the only argument is func, so it's not a big issue
    """

    def new_decorator(func):
        # Call to newDecorator(method)
        # Exactly like old decorator, but output keeps track of what decorated it
        output = decorator(
            func
        )  # apply foreignDecorator, like call to foreignDecorator(method) would have done
        output.decorator = new_decorator  # keep track of decorator
        # R.original = func         # might as well keep track of everything!
        return output

    new_decorator.__name__ = decorator.__name__
    new_decorator.__doc__ = decorator.__doc__
    new_decorator.__is_endpoint__ = True

    return new_decorator


# https://stackoverflow.com/questions/2366713/can-a-decorator-of-an-instance-method-access-the-class
# noinspection PyUnusedLocal
def endpoint(verb: str = None, path: str = None, **kwargs):
    """By using **kw we can tag the function with Any parameters"""

    def decorator(function):
        # This is used in conjunction with the __init_subclass__ code!
        # Otherwise the __name__ won't be correct in maybeDecorated.__name__!
        # noinspection PyShadowingNames
        @wraps(function)
        def wrap(self, *args, **kwargs):
            return function(self, *args, **kwargs)

        wrap.__path__ = path
        wrap.__verb__ = verb
        return wrap

    decorator = make_registering_decorator(decorator)
    return decorator


def get(path: str, **kwargs):
    return endpoint(verb=Verb.GET, path=path, **kwargs)


def post(path: str, **kwargs):
    return endpoint(verb=Verb.POST, path=path, **kwargs)


class App:
    """A Steamship microservice.

    This base.py class:

      1. Provide a pre-authenticated instance of the Steamship client
      2. Provides a Lambda handler that routes to registered functions
      3. Provides useful methods connecting functions to the router.
    """

    _method_mappings = defaultdict(dict)

    def __init__(self, client: Client = None, config: Dict[str, Any] = None):
        self.client = client
        self.config = config

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls._method_mappings = defaultdict(dict)

        for maybeDecorated in cls.__dict__.values():
            decorator = getattr(maybeDecorated, "decorator", None)
            if decorator:
                if getattr(decorator, "__is_endpoint__", False):
                    path = getattr(maybeDecorated, "__path__", None)
                    verb = getattr(maybeDecorated, "__verb__", None)
                    cls._register_mapping(maybeDecorated.__name__, verb, path)

    @staticmethod
    def _clean_path(path: str = "") -> str:
        if not path:
            path = "/"
        elif path[0] != "/":
            path = f"/{path}"
        return path

    @classmethod
    def _register_mapping(
        cls, name: str, verb: Optional[Verb] = None, path: str = ""
    ) -> None:
        """Registering a mapping permits the method to be invoked via HTTP."""

        verb = verb or Verb.GET

        if path is None and name is not None:
            path = f"/{name}"

        path = cls._clean_path(path)

        cls._method_mappings[verb][path] = name
        logging.info(f"[{cls.__name__}] {verb} {path} => {name}")

    def __call__(self, request: Request, context: Any = None) -> Response:
        """Invokes a method call if it is registered."""
        if not getattr(self.__class__, "_method_mappings"):
            return Response.error(
                code=HTTPStatus.NOT_FOUND, message="No mappings available for app."
            )

        if request.invocation is None:
            return Response.error(
                code=HTTPStatus.NOT_FOUND, message="No invocation was found."
            )

        verb = Verb.safely_from_str(request.invocation.httpVerb)
        path = request.invocation.appPath

        path = self._clean_path(path)

        method_mappings = self.__class__._method_mappings

        if verb not in method_mappings:
            return Response.error(
                code=HTTPStatus.NOT_FOUND,
                message=f"No methods for verb {verb} available.",
            )

        if path not in method_mappings[verb]:
            return Response.error(
                code=HTTPStatus.NOT_FOUND,
                message=f"No handler for {verb} {path} available.",
            )

        method = method_mappings[verb][path]
        if not (hasattr(self, method) and callable(getattr(self, method))):
            return Response.error(
                code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Handler for {verb} {path} not callable.",
            )

        arguments = request.invocation.arguments
        if arguments is None:
            return getattr(self, method)()
        else:
            return getattr(self, method)(**arguments)



from steamship.base import Client
from steamship.base.response import TaskState
from steamship.data.embeddings import EmbeddedItem
from steamship.data.plugin_instance import PluginInstance

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client
from tests.utils.random import random_index

_TEST_EMBEDDER = "test-embedder"


def create_index(_: Client, plugin_instance: str):
    steamship = get_steamship_client()

    # Should require plugin
    task = steamship.create_index()
    assert task.error is not None
    assert task.data is None

    index = steamship.create_index(plugin_instance=plugin_instance).data
    assert index is not None

    # Duplicate creation should fail with upsert=False
    task = steamship.create_index(
        handle=index.handle, plugin_instance=plugin_instance, upsert=False
    )
    assert task.error is not None
    assert task.data is None

    index.delete()


def test_create_index():
    client = get_steamship_client()
    plugin_instance = PluginInstance.create(client, plugin_handle=_TEST_EMBEDDER).data
    create_index(client, plugin_instance.handle)


def test_delete_index():
    steamship = get_steamship_client()
    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    index = steamship.create_index(plugin_instance=plugin_instance.handle).data
    assert index.id is not None

    task = steamship.create_index(
        handle=index.handle, plugin_instance=plugin_instance.handle
    )
    assert task.error is None
    index2 = task.data
    assert index.id == index2.id

    index.delete()

    task = steamship.create_index(plugin_instance=plugin_instance.handle)
    assert task.error is None
    assert task.data is not None
    index3 = task.data
    assert index.id != index3.id
    index3.delete()


def _list_equal(actual, expected):
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_insert_many():
    steamship = get_steamship_client()
    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    with random_index(steamship, plugin_instance.handle) as index:
        item1 = EmbeddedItem(
            value="Pizza", externalId="pizza", externalType="food", metadata=[1, 2, 3]
        )
        item2 = EmbeddedItem(
            value="Rocket Ship",
            externalId="space",
            externalType="vehicle",
            metadata="Foo",
        )

        index.insert_many([item1, item2])
        index.embed().wait()

        task = index.list_items()
        assert task.error is None
        index_items = task.data
        assert len(index_items.items) == 2
        assert len(index_items.items[0].embedding) > 0
        assert len(index_items.items[1].embedding) > 0
        assert len(index_items.items[0].embedding) == len(
            index_items.items[1].embedding
        )

        res = index.search(item1.value, include_metadata=True, k=100)
        assert res.data.items is not None
        assert len(res.data.items) == 2
        assert res.data.items[0].value.value == item1.value
        assert res.data.items[0].value.external_id == item1.externalId
        assert res.data.items[0].value.external_type == item1.externalType
        _list_equal(res.data.items[0].value.metadata, item1.metadata)

        res = index.search(item2.value, include_metadata=True)
        assert res.data.items is not None
        assert res.data.items[0].value.value == item2.value
        assert res.data.items[0].value.external_id == item2.externalId
        assert res.data.items[0].value.external_type == item2.externalType
        assert res.data.items[0].value.metadata == item2.metadata


def test_embed_task():
    steamship = get_steamship_client()
    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    with random_index(steamship, plugin_instance.handle) as index:
        _ = index.insert("test", reindex=False)
        res = index.embed()

        assert res.task.task_id is not None
        assert res.task.state is not None
        assert res.task.task_created_on is not None
        assert res.task.task_last_modified_on is not None
        assert res.task.state == TaskState.waiting
        res.wait()
        assert res.task.state == TaskState.succeeded


def test_duplicate_inserts():
    steamship = get_steamship_client()

    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    with random_index(steamship, plugin_instance.handle) as index:
        # Test for suppressed re-indexing
        a1 = "Ted can eat an entire block of cheese."
        q1 = "Who can eat the most cheese"
        _ = index.insert(a1)
        _ = index.search(q1)


def test_index_usage():
    steamship = get_steamship_client()

    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    with random_index(steamship, plugin_instance.handle) as index:
        a1 = "Ted can eat an entire block of cheese."
        q1 = "Who can eat the most cheese"
        _ = index.insert(a1)
        _ = index.search(q1)

        # Now embed
        task = index.embed()
        task.wait()
        task.check()
        assert task.task.state == TaskState.succeeded

        search_results = index.search(q1)
        assert len(search_results.data.items) == 1
        assert search_results.data.items[0].value.value == a1

        # Associate metadata
        a2 = "Armadillo shells are bulletproof."
        q2 = "What is something interesting about Armadillos?"
        a2id = "A2id"
        a2type = "A2type"
        a2metadata = dict(
            id=a2id,
            idid=f"{a2id}{a2id}",
            boolVal=True,
            intVal=123,
            floatVal=1.2,
        )

        _ = index.insert(
            a2, external_id=a2id, external_type=a2type, metadata=a2metadata
        )
        search_results2 = index.search(q2)
        assert len(search_results2.data.items) == 1
        assert search_results2.data.items[0].value.value == a2
        assert search_results2.data.items[0].value.external_id is None
        assert search_results2.data.items[0].value.external_type is None
        assert search_results2.data.items[0].value.metadata is None

        search_results3 = index.search(q2, include_metadata=True)
        assert len(search_results3.data.items) == 1
        assert search_results3.data.items[0].value.value == a2
        assert search_results3.data.items[0].value.external_id == a2id
        assert search_results3.data.items[0].value.external_type == a2type

        assert search_results3.data.items[0].value.metadata == a2metadata
        # Because I don't know pytest enough to fully trust the dict comparison..
        assert search_results3.data.items[0].value.metadata["id"] == a2id
        assert search_results3.data.items[0].value.metadata["idid"] == "{}{}".format(
            a2id, a2id
        )

        search_results4 = index.search(q2, k=10)
        assert len(search_results4.data.items) == 2
        assert search_results4.data.items[0].value.value == a2
        assert search_results4.data.items[1].value.value == a1


def test_multiple_queries():
    steamship = get_steamship_client()

    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    with random_index(steamship, plugin_instance.handle) as index:
        # Test for suppressed re-indexing
        a1 = "Ted can eat an entire block of cheese."
        a2 = "Joe can drink an entire glass of water."
        _ = index.insert_many([a1, a2])
        index.embed().wait()

        qs1 = ["Who can eat the most cheese", "Who can run the fastest?"]
        search_results = index.search(qs1)
        assert len(search_results.data.items) == 1
        assert search_results.data.items[0].value.value == a1
        assert search_results.data.items[0].value.query == qs1[0]

        qs2 = ["Who can tie a shoe?", "Who can drink the most water?"]
        search_results = index.search(qs2)
        assert len(search_results.data.items) == 1
        assert search_results.data.items[0].value.value == a2
        assert search_results.data.items[0].value.query == qs2[1]

        qs3 = ["What can Ted do?", "What can Sam do?", "What can Jerry do?"]
        search_results = index.search(qs3)
        assert len(search_results.data.items) == 1
        assert search_results.data.items[0].value.value == a1
        assert search_results.data.items[0].value.query == qs3[0]

        qs3 = ["What can Sam do?", "What can Ted do?", "What can Jerry do?"]
        search_results = index.search(qs3)
        assert len(search_results.data.items) == 1
        assert search_results.data.items[0].value.value == a1
        assert search_results.data.items[0].value.query == qs3[1]

        index.create_snapshot().wait()

        a3 = "Susan can run very fast."
        a4 = "Brenda can fight alligators."
        _ = index.insert_many([a3, a4])
        index.embed().wait()

        qs4 = ["What can Brenda do?", "What can Ronaldo do?", "What can Jerry do?"]
        search_results = index.search(qs4)
        assert len(search_results.data.items) == 1
        assert search_results.data.items[0].value.value == a4
        assert search_results.data.items[0].value.query == qs4[0]

        qs4 = [
            "What can Brenda do?",
            "Who should run a marathon?",
            "What can Jerry do?",
        ]
        search_results = index.search(qs4, k=2)
        assert len(search_results.data.items) == 2
        assert search_results.data.items[0].value.value == a4
        assert search_results.data.items[0].value.query == qs4[0]
        assert search_results.data.items[1].value.value == a3
        assert search_results.data.items[1].value.query == qs4[1]


def test_empty_queries():
    steamship = get_steamship_client()

    plugin_instance = PluginInstance.create(
        steamship, plugin_handle=_TEST_EMBEDDER
    ).data
    with random_index(steamship, plugin_instance.handle) as index:
        a1 = "Ted can eat an entire block of cheese."
        a2 = "Joe can drink an entire glass of water."
        _ = index.insert_many([a1, a2])
        index.embed().wait()

        search_results = index.search(None)
        assert search_results.error is not None
        assert search_results.data is None

        # These technically don't count as empty. Leaving this test in here
        # to encode and capture that in case we want to change it.
        search_results = index.search([])
        assert len(search_results.data.items) == 0

        search_results = index.search("")
        assert len(search_results.data.items) == 1



# from os import path
#
# from steamship import MimeTypes
#
# from .helpers import _random_name, _steamship
#
# __copyright__ = "Steamship"
# __license__ = "MIT"
#
#
# def test_image_upload():
#     steamship = get_steamship_client()
#     test_filename = path.join(
#         path.dirname(path.realpath(__file__)),
#         'test_img.png'
#     )
#     with open(test_filename, "rb") as f:
#         name_c = "{}.png".format(_random_name())
#         c = steamship.upload(
#             name=name_c,
#             content=f
#         ).data
#         assert (c.id is not None)
#         assert (c.name == name_c)
#         assert (c.mimeType == MimeTypes.PNG)
#
#         blockifyResp = c.blockify(plugin="ocr_ms_vision_default")
#         assert (blockifyResp.error is None)
#         blockifyResp.wait()
#         assert (blockifyResp.data is not None)
#
#         q1 = c.query().data
#         assert (len(q1.blocks) == 3)
#
#         c.delete()



from typing import Any

from steamship import Corpus
from steamship.app import Response
from steamship.plugin.corpus_importer import CorpusImportRequest, CorpusImportResponse


def do_import(
    self,
    value: str = None,
    url: str = None,
    plugin_instance: str = None,
    space_id: str = None,
    space_handle: str = None,
    space: Any = None,
    file_importer_plugin_instance: str = None,
) -> "Response[CorpusImportResponse]":
    # TODO (enias): Why not part of Corpus?
    req = CorpusImportRequest(
        type="corpus",
        id=self.id,
        value=value,
        url=url,
        pluginInstance=plugin_instance,
        fileImporterPluginInstance=file_importer_plugin_instance,
    )
    return self.client.post(
        "plugin/instance/importCorpus",
        req,
        expect=CorpusImportResponse,
        space_id=space_id,
        space_handle=space_handle,
        space=space,
    )


Corpus.do_import = do_import



import base64

import requests

from steamship.base.mime_types import MimeTypes
from tests import APPS_PATH, TEST_ASSETS_PATH

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client
from tests.utils.deployables import deploy_app


def test_instance_invoke():
    palm_tree_path = TEST_ASSETS_PATH / "palm_tree.png"

    with palm_tree_path.open("rb") as f:
        palm_bytes = f.read()
    base64_palm = base64.b64encode(palm_bytes).decode("utf-8")

    client = get_steamship_client()
    demo_app_path = APPS_PATH / "apps" / "demo_app.py"

    with deploy_app(client, demo_app_path) as (app, version, instance):
        # Now let's invoke it!
        # Note: we're invoking the data at demo_app.py in the tests/demo_apps folder

        def get_raw(path: str):
            return requests.get(
                instance.full_url_for(path),
                headers=dict(authorization=f"Bearer {client.config.api_key}"),
            )

        # res = instance.get("greet").data
        # assert res == "Hello, Person!"
        #
        # resp = get_raw("greet")
        # assert resp.text == "Hello, Person!"
        #
        # res = instance.get("greet", name="Ted").data
        # assert res == "Hello, Ted!"
        # url = instance.full_url_for("greet?name=Ted")
        # resp = requests.get(
        #     url, headers=dict(authorization=f"Bearer {client.config.api_key}")
        # )
        # assert resp.text == "Hello, Ted!"
        #
        # res = instance.post("greet").data
        # assert res == "Hello, Person!"
        # url = instance.full_url_for("greet")
        # resp = requests.post(
        #     url, headers=dict(authorization=f"Bearer {client.config.api_key}")
        # )
        # assert resp.text == "Hello, Person!"
        #
        # res = instance.post("greet", name="Ted").data
        # assert res == "Hello, Ted!"
        # url = instance.full_url_for("greet")
        # resp = requests.post(
        #     url,
        #     json=dict(name="Ted"),
        #     headers=dict(authorization=f"Bearer {client.config.api_key}"),
        # )
        # assert resp.text == "Hello, Ted!"
        #
        # # Now we test different return types
        # resp_string = get_raw("resp_string")
        # assert resp_string.text == "A String"
        #
        # resp_dict = get_raw("resp_dict")
        # assert resp_dict.json() == dict(string="A String", int=10)

        # resp_404 = get_raw("doesnt_exist")
        # json_404 = resp_404.json()
        # assert isinstance(json_404, dict)
        # assert json_404.get("status", None) is not None
        # assert json_404.get("status", None) is not None
        # assert json_404.get("status", dict()).get("state", None) == TaskState.failed
        # # assert "No handler" in json_404.get("status", dict()).get("statusMessage", "")
        # assert resp_404.status_code == 404
        #
        # resp_obj = get_raw("resp_obj")
        # assert resp_obj.json() == dict(name="Foo")

        resp_binary = get_raw("resp_binary")
        base64_binary = base64.b64encode(resp_binary.content).decode("utf-8")
        assert base64_binary == base64_palm
        assert resp_binary.headers.get("Content-Type") == MimeTypes.BINARY

        resp_bytes_io = get_raw("resp_bytes_io")
        base64_bytes_io = base64.b64encode(resp_bytes_io.content).decode("utf-8")
        assert base64_bytes_io == base64_palm
        assert resp_bytes_io.headers.get("Content-Type") == MimeTypes.BINARY

        resp_image = get_raw("resp_image")
        base64_image = base64.b64encode(resp_image.content).decode("utf-8")
        assert base64_image == base64_palm
        assert resp_image.headers.get("Content-Type") == MimeTypes.PNG



import contextlib

from steamship import File, Steamship
from tests import TEST_ASSETS_PATH


@contextlib.contextmanager
def upload_file(client: Steamship, test_asset_filename: str):
    full_path = TEST_ASSETS_PATH / test_asset_filename
    file = File.create(client, filename=str(full_path))
    assert file.data is not None
    assert file.data.id is not None
    yield file.data
    file.data.delete()



from dataclasses import dataclass
from typing import Any, Dict, List

from steamship.base import Client, Request
from steamship.base.response import Response
from steamship.data.plugin import ListPluginsResponse


class PluginVersion:
    pass


@dataclass
class CreatePluginVersionRequest(Request):
    pluginId: str = None
    handle: str = None
    upsert: bool = None
    hostingMemory: str = None
    hostingTimeout: str = None
    hostingHandler: str = None
    isPublic: bool = None
    isDefault: bool = None
    type: str = "file"
    configTemplate: Dict[str, Any] = None


@dataclass
class DeletePluginVersionRequest(Request):
    id: str


@dataclass
class ListPublicPluginVersionsRequest(Request):
    handle: str
    pluginId: str


@dataclass
class ListPrivatePluginVersionsRequest(Request):
    handle: str
    pluginId: str


@dataclass
class ListPluginVersionsResponse(Request):
    plugins: List[PluginVersion]

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "ListPluginsResponse":
        return ListPluginVersionsResponse(
            plugins=[
                PluginVersion.from_dict(x, client=client)
                for x in (d.get("pluginVersions", []) or [])
            ]
        )


@dataclass
class PluginVersion:
    client: Client = None
    id: str = None
    pluginId: str = None
    handle: str = None
    hostingMemory: str = None
    hostingTimeout: str = None
    hostingHandler: str = None
    isPublic: bool = None
    isDefault: bool = None
    configTemplate: Dict[str, Any] = None

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "PluginVersion":
        if "pluginVersion" in d:
            d = d["pluginVersion"]

        return PluginVersion(
            client=client,
            id=d.get("id", None),
            handle=d.get("handle", None),
            hostingMemory=d.get("hostingMemory", None),
            hostingTimeout=d.get("hostingTimeout", None),
            hostingHandler=d.get("hostingHandler", None),
            isPublic=d.get("isPublic", None),
            isDefault=d.get("isDefault", None),
            configTemplate=d.get("configTemplate", None),
        )

    @staticmethod
    def create(
        client: Client,
        handle: str,
        plugin_id: str = None,
        filename: str = None,
        filebytes: bytes = None,
        upsert: bool = None,
        hosting_memory: str = None,
        hosting_timeout: str = None,
        hosting_handler: str = None,
        is_public: bool = None,
        is_default: bool = None,
        config_template: Dict[str, Any] = None,
    ) -> Response[PluginVersion]:

        if filename is None and filebytes is None:
            raise Exception("Either filename or filebytes must be provided.")
        if filename is not None and filebytes is not None:
            raise Exception("Only either filename or filebytes should be provided.")

        if filename is not None:
            with open(filename, "rb") as f:
                filebytes = f.read()

        req = CreatePluginVersionRequest(
            handle=handle,
            pluginId=plugin_id,
            upsert=upsert,
            hostingMemory=hosting_memory,
            hostingTimeout=hosting_timeout,
            hostingHandler=hosting_handler,
            isPublic=is_public,
            isDefault=is_default,
            configTemplate=config_template,
        )

        return client.post(
            "plugin/version/create",
            payload=req,
            file=("plugin.zip", filebytes, "multipart/form-data"),
            expect=PluginVersion,
        )

    def delete(self) -> "PluginVersion":
        req = DeletePluginVersionRequest(id=self.id)
        return self.client.post(
            "plugin/version/delete", payload=req, expect=PluginVersion
        )

    @staticmethod
    def get_public(client: Client, plugin_id: str, handle: str):
        public_plugins = PluginVersion.list_public(
            client=client, plugin_id=plugin_id, handle=handle
        ).data.plugins
        for plugin in public_plugins:
            if plugin.handle == handle:
                return plugin
        return None

    @staticmethod
    def list_public(
        client: Client, plugin_id: str = None, handle: str = None
    ) -> Response[ListPluginVersionsResponse]:
        return client.post(
            "plugin/version/public",
            ListPublicPluginVersionsRequest(handle=handle, pluginId=plugin_id),
            expect=ListPluginVersionsResponse,
        )

    @staticmethod
    def list_private(
        client: Client, _=None, plugin_id: str = None, handle: str = None
    ) -> Response[ListPluginVersionsResponse]:
        return client.post(
            "plugin/version/private",
            ListPrivatePluginVersionsRequest(handle=handle, pluginId=plugin_id),
            expect=ListPluginVersionsResponse,
        )



import logging

from steamship.base import Client, Response, TaskComment
from steamship.base.tasks import TaskCommentList

__copyright__ = "Steamship"
__license__ = "MIT"

_logger = logging.getLogger(__name__)


class Tasks:
    """Asynchronous background task (and task feedback) management."""

    def __init__(self, client: Client):
        self.client = client

    def list_comments(
        self,
        task_id: str = None,
        external_id: str = None,
        external_type: str = None,
        external_group: str = None,
    ) -> Response[TaskCommentList]:
        return TaskComment.list(
            client=self.client,
            task_id=task_id,
            external_id=external_id,
            external_type=external_type,
            external_group=external_group,
        )



from steamship.extension.file import File

__copyright__ = "Steamship"
__license__ = "MIT"

from tests import APPS_PATH
from tests.utils.client import get_steamship_client
from tests.utils.deployables import deploy_plugin


def test_e2e_blockifier_plugin():
    client = get_steamship_client()
    blockifier_path = APPS_PATH / "plugins" / "blockifiers" / "blockifier.py"
    with deploy_plugin(client, blockifier_path, "blockifier") as (
        plugin,
        version,
        instance,
    ):
        file = File.create(client=client, content="This is a test.").data
        assert len(file.refresh().data.blocks) == 0
        file.blockify(plugin_instance=instance.handle).wait()
        assert len(file.refresh().data.blocks) == 4
        file.delete()



# import pytest
# import os
# import random
# import string
# import contextlib

# from steamship import Steamship, ClassifierModels, Classifier
# from .helpers import _random_index, _random_name, _steamship

# __copyright__ = "Steamship"
# __license__ = "MIT"

# def test_classifier_create():
#   steamship = get_steamship_client()

#   # Should require name
#   with pytest.raises(Exception):
#     index = steamship.create_classifier(
#       plugin=ClassifierModels.HF_ZERO_SHOT_LBART
#     )

#   # Should require plugin
#   with pytest.raises(Exception):
#     index = steamship.create_classifier(
#       name="Test Index"
#     )

#   classifier = steamship.create_classifier(
#     name="Test 2",
#     plugin=ClassifierModels.HF_ZERO_SHOT_LBART,
#     save=False
#   ).data
#   assert classifier is not None

#   # Missing labels as a zero shot
#   with pytest.raises(Exception):
#       results = classifier.classify(docs=["Jurassic Park"])

#   # Missing docs
#   with pytest.raises(Exception):
#       results = classifier.classify(labels=["Jurassic Park"])

#   # Create one and let's use it
#   classifier = steamship.create_classifier(
#     name="Fruits",
#     plugin=ClassifierModels.HF_ZERO_SHOT_LBART,
#     save=False
#   ).data
#   assert classifier is not None
#   r = classifier.classify(docs=["Banana"], labels=["Movie", "Food", "City"], k=2)
#   assert len(r.data.hits) == 1
#   assert len(r.data.hits[0]) == 2
#   assert r.data.hits[0][0].value == "Food"

#   r = classifier.classify(docs=["Banana"], labels=["Movie", "Food", "City"], k=3)

#   assert len(r.data.hits) == 1
#   assert len(r.data.hits[0]) == 3
#   assert r.data.hits[0][0].value == "Food"

#   r = classifier.classify(docs=["Banana", "Boston"], labels=["Movie", "Food", "City"], k=3)
#   assert len(r.data.hits) == 2
#   assert len(r.data.hits[0]) == 3
#   assert r.data.hits[0][0].value == "Food"
#   assert len(r.data.hits[1]) == 3
#   assert r.data.hits[1][0].value == "City"



import json
from dataclasses import dataclass
from typing import Any

from steamship.base import Client


@dataclass
class Hit:
    id: str = None
    index: int = None
    index_source: str = None
    value: str = None
    score: float = None
    external_id: str = None
    external_type: str = None
    metadata: Any = None
    query: str = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "Hit":
        metadata = d.get("metadata")
        if metadata is not None:
            # noinspection PyBroadException
            try:
                metadata = json.loads(metadata)
            except Exception as _:
                pass

        return Hit(
            id=d.get("id"),
            index=d.get("index"),
            index_source=d.get("indexSource"),
            value=d.get("value", d.get("text")),
            score=d.get("score"),
            external_id=d.get("externalId"),
            external_type=d.get("externalType"),
            metadata=metadata,
            query=d.get("query"),
        )



from dataclasses import dataclass
from typing import Any, Dict

from steamship.base import Client
from steamship.plugin.inputs.export_plugin_input import ExportPluginInput


@dataclass
class TrainingParameterPluginOutput:
    machineType: str = None
    modelName: str = None
    modelFilename: str = None

    trainingEpochs: int = None
    testingHoldoutPercent: float = None
    testSplitSeed: int = None
    trainingParams: Dict[str, Any] = None

    exportRequest: ExportPluginInput = None

    @staticmethod
    def from_dict(
        d: Any = None, client: Client = None
    ) -> "TrainingParameterPluginOutput":
        if d is None:
            return None

        return TrainingParameterPluginOutput(
            machineType=d.get("machineType", None),
            modelName=d.get("modelName", None),
            modelFilename=d.get("modelFilename", None),
            trainingEpochs=d.get("trainingEpochs", None),
            testingHoldoutPercent=d.get("testingHoldoutPercent", None),
            testSplitSeed=d.get("testSplitSeed", None),
            trainingParams=d.get("trainingParams", None),
            exportRequest=ExportPluginInput.from_dict(
                d.get("exportPluginInput", None), client
            ),
        )

    def to_dict(self) -> Dict:
        export_plugin_input_params = None
        if self.exportRequest is not None:
            export_plugin_input_params = self.exportRequest.to_dict()

        return dict(
            machineType=self.machineType,
            modelName=self.modelName,
            modelFilename=self.modelFilename,
            trainingEpochs=self.trainingEpochs,
            testingHoldoutPercent=self.testingHoldoutPercent,
            testSplitSeed=self.testSplitSeed,
            trainingParams=self.trainingParams,
            exportRequest=export_plugin_input_params,
        )



from steamship import File, PluginInstance, Steamship

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client

_TEST_EMBEDDER = "test-embedder"


def count_embeddings(file: File):
    embeddings = 0
    for block in file.blocks:
        for tag in block.tags:
            if tag.kind == "text" and tag.name == "embedding":
                embeddings += 1
    return embeddings


def basic_embeddings(steamship: Steamship, plugin_instance: str):
    e1 = steamship.tag("This is a test", plugin_instance=plugin_instance)
    e1b = steamship.tag("Banana", plugin_instance=plugin_instance)
    e1.wait()
    e1b.wait()
    assert count_embeddings(e1.data.file) == 1
    assert count_embeddings(e1b.data.file) == 1
    assert len(e1.data.file.blocks[0].tags[0].value["embedding"]) > 1

    e2 = steamship.tag("This is a test", plugin_instance=plugin_instance)
    e2.wait()
    assert count_embeddings(e2.data.file) == 1
    assert len(e2.data.file.blocks[0].tags[0].value["embedding"]) == len(
        e1.data.file.blocks[0].tags[0].value["embedding"]
    )

    e4 = steamship.tag("This is a test", plugin_instance=plugin_instance)
    e4.wait()
    assert count_embeddings(e4.data.file) == 1


def test_basic_embeddings():
    client = get_steamship_client()
    plugin_instance = PluginInstance.create(client, plugin_handle=_TEST_EMBEDDER).data
    basic_embeddings(client, plugin_instance.handle)


def basic_embedding_search(steamship: Steamship, plugin_instance: str):
    docs = [
        "Armadillo shells are bulletproof.",
        "Dolphins sleep with one eye open.",
        "Alfred Hitchcock was frightened of eggs.",
        "Jonathan can help you with new employee onboarding",
        "The code for the New York office is 1234",
    ]
    query = "Who should I talk to about new employee setup?"
    results = steamship.embed_and_search(query, docs, plugin_instance=plugin_instance)
    assert len(results.data.items) == 1
    assert (
        results.data.items[0].value.value
        == "Jonathan can help you with new employee onboarding"
    )


def test_basic_embedding_search():
    client = get_steamship_client()
    plugin_instance = PluginInstance.create(client, plugin_handle=_TEST_EMBEDDER).data
    basic_embedding_search(client, plugin_instance.handle)



from dataclasses import dataclass
from typing import Any, Dict, Optional

from steamship.base import Client


@dataclass
class FileImportPluginInput:
    value: str = None
    data: str = None
    url: str = None
    pluginInstance: str = None
    mimeType: str = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(
        d: Any = None, client: Client = None
    ) -> "Optional[FileImportPluginInput]":
        if d is None:
            return None

        return FileImportPluginInput(
            value=d.get("value", None),
            data=d.get("data", None),
            url=d.get("url", None),
            pluginInstance=d.get("pluginInstance", None),
            mimeType=d.get("mimeType", None),
        )

    def to_dict(self) -> Dict:
        # TODO (enias): Debug why we reference a non-existent variable file
        raise NotImplementedError()



import dataclasses
import json

__copyright__ = "Steamship"
__license__ = "MIT"

from steamship.base.binary_utils import flexi_create


def test_dump_string():
    data, mime, encoding = flexi_create(string="Hi")
    assert (data == "Hi")


def test_dump_json():
    assert (json.dumps(flexi_create(json="Hi")[0]) == '"Hi"')
    assert (json.dumps(flexi_create(json=True)[0]) == 'true')
    assert (json.dumps(flexi_create(json=1.2)[0]) == '1.2')
    assert (json.dumps(flexi_create(json=[1, 2, 3])[0]) == '[1, 2, 3]')
    assert (json.dumps(flexi_create(json={'hi': 'there'})[0]) == '{"hi": "there"}')

    @dataclasses.dataclass
    class Person:
        name: str

    person = Person(name="Ted")
    assert (json.dumps(flexi_create(json=person)[0]) == '{"name": "Ted"}')

    @dataclasses.dataclass
    class Person2:
        name: str

        def to_dict(self):
            return {"takes": "precedence"}

    person2 = Person2(name="Ted")
    assert (json.dumps(flexi_create(json=person2)[0]) == '{"takes": "precedence"}')



from steamship import Steamship
from steamship.data.user import User


def get_steamship_client() -> Steamship:
    # TODO: The asserts do not have to be run every time
    # This should automatically pick up variables from the environment.
    client = Steamship(profile="test")
    assert client.config is not None
    assert client.config.profile == "test"
    assert client.config.api_key is not None
    user = User.current(client).data
    assert user.id is not None
    assert user.handle is not None
    return client



from tests.client.operations.test_tag_file import tag_file

__copyright__ = "Steamship"
__license__ = "MIT"

from .. import APPS_PATH
from ..utils.client import get_steamship_client
from ..utils.deployables import deploy_plugin


def test_e2e_tagger():
    client = get_steamship_client()
    parser_path = APPS_PATH / "plugins" / "taggers" / "plugin_parser.py"
    # TODO (enias): Use Enum for plugin type
    with deploy_plugin(client, parser_path, "tagger") as (plugin, version, instance):
        test_doc = "Hi there"
        res = client.tag(doc=test_doc, plugin_instance=instance.handle)
        res.wait()
        assert res.error is None
        assert res.data is not None
        assert len(res.data.file.blocks) == 1
        assert res.data.file.blocks[0].text == test_doc

        # Let's try it on a file. This is the same test we run on the Swift test parser.
        # Since the python test parser is implemented to behave the same, we can reuse it!
        tag_file(client, instance.handle)



from dataclasses import dataclass
from typing import Any, Dict

from steamship.base import Client
from steamship.plugin.inputs.export_plugin_input import ExportPluginInput


@dataclass
class TrainingParameterPluginInput:
    pluginInstance: str = None
    exportRequest: ExportPluginInput = None

    @staticmethod
    def from_dict(
        d: Any = None, client: Client = None
    ) -> "TrainingParameterPluginInput":
        if d is None:
            return None

        return TrainingParameterPluginInput(
            pluginInstance=d.get("pluginInstance", None),
            exportRequest=ExportPluginInput.from_dict(
                d.get("exportPluginInput", None), client
            ),
        )

    def to_dict(self) -> Dict:
        export_plugin_input_params = None
        if self.exportRequest is not None:
            export_plugin_input_params = self.exportRequest.to_dict()

        return dict(
            pluginInstance=self.pluginInstance,
            exportPluginInput=export_plugin_input_params,
        )



from dataclasses import dataclass
from typing import Any, Dict

from steamship.base import Client, Request, Response
from steamship.data.space import Space


@dataclass
class CreateAppInstanceRequest(Request):
    id: str = None
    appId: str = None
    appVersionId: str = None
    handle: str = None
    upsert: bool = None
    config: Dict[str, Any] = None


@dataclass
class DeleteAppInstanceRequest(Request):
    id: str


@dataclass
class AppInstance:
    client: Client = None
    id: str = None
    handle: str = None
    app_id: str = None
    app_handle: str = None
    user_handle: str = None
    app_version_id: str = None
    user_id: str = None
    invocation_url: str = None
    config: Dict[str, Any] = None
    space_id: str = None
    space_handle: str = None

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "AppInstance":
        if "appInstance" in d:
            d = d["appInstance"]

        return AppInstance(
            client=client,
            id=d.get("id"),
            handle=d.get("handle"),
            app_id=d.get("appId"),
            app_handle=d.get("appHandle"),
            user_handle=d.get("userHandle"),
            app_version_id=d.get("appVersionId"),
            user_id=d.get("userId"),
            invocation_url=d.get("invocationURL"),
            config=d.get("config"),
            space_id=d.get("spaceId"),
            space_handle=d.get("spaceHandle"),
        )

    @staticmethod
    def create(
        client: Client,
        app_id: str = None,
        app_version_id: str = None,
        handle: str = None,
        upsert: bool = None,
        config: Dict[str, Any] = None,
    ) -> "Response[AppInstance]":

        req = CreateAppInstanceRequest(
            handle=handle,
            appId=app_id,
            appVersionId=app_version_id,
            upsert=upsert,
            config=config,
        )

        return client.post("app/instance/create", payload=req, expect=AppInstance)

    def delete(self) -> "AppInstance":
        req = DeleteAppInstanceRequest(id=self.id)
        return self.client.post("app/instance/delete", payload=req, expect=AppInstance)

    def load_missing_vals(self):
        if (
            self.client is not None
            and self.space_handle is None
            and self.space_id is not None
        ):
            # Get the spaceHandle
            space = Space.get(self.client, id_=self.space_id)
            if space and space.data:
                self.space_handle = space.data.handle

    def get(self, path: str, **kwargs):
        self.load_missing_vals()
        if path[0] == "/":
            path = path[1:]
        return self.client.get(
            f"/{self.space_handle or '_'}/{self.handle or '_'}/{path}",  # TODO (enias): Fix code duplication
            payload=kwargs,
            app_call=True,
            app_owner=self.user_handle,
            app_id=self.app_id,
            app_instance_id=self.id,
            space_id=self.space_id,
        )

    def post(self, path: str, **kwargs):
        self.load_missing_vals()
        if path[0] == "/":
            path = path[1:]
        return self.client.post(
            f"/{self.space_handle or '_'}/{self.handle or '_'}/{path}",
            payload=kwargs,
            app_call=True,
            app_owner=self.user_handle,
            app_id=self.app_id,
            app_instance_id=self.id,
            space_id=self.space_id,
        )

    def full_url_for(self, path: str):
        return f"{self.invocation_url}{path}"


@dataclass
class ListPrivateAppInstancesRequest(Request):
    pass



from dataclasses import asdict
from typing import Any

from steamship.app.response import Response
from steamship.base.mime_types import MimeTypes
from steamship.data.file import File
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput

__copyright__ = "Steamship"
__license__ = "MIT"


def check_mime(d: dict, mime: str):
    assert (
        d.get("http", dict()).get("headers", dict()).get("Content-Type", None) == mime
    )


def check_type(d: dict, t: Any):
    data = d.get("data", None)
    assert isinstance(data, t)


def check_val(d: dict, val: Any):
    data = d.get("data", None)
    assert data == val


def test_text_response():
    """Test test response"""
    r = Response.from_obj("Hi there")
    d = r.to_dict()
    check_mime(d, MimeTypes.TXT)
    check_type(d, str)
    check_val(d, "Hi there")


def test_dict_response():
    """Test test response"""
    r = Response.from_obj(dict(a=1))
    d = r.to_dict()
    check_mime(d, MimeTypes.JSON)
    check_type(d, dict)
    check_val(d, dict(a=1))


def test_resp_response():
    """Test test response"""
    o = BlockAndTagPluginOutput(file=File.CreateRequest(value="Foo", blocks=[], tags=[]))
    r = Response(json=o)
    d = r.to_dict()
    check_mime(d, MimeTypes.JSON)
    check_type(d, dict)
    check_val(d, asdict(o))



from dataclasses import dataclass
from typing import Any, List

from steamship.base import Client


@dataclass
class EmbeddedItemsPluginOutput:
    embeddings: List[List[float]]

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "EmbeddedItemsPluginOutput":
        return EmbeddedItemsPluginOutput(
            embeddings=d.get("embeddings", None),
        )

    def to_dict(self) -> dict:
        return dict(embeddings=self.embeddings)



from dataclasses import dataclass
from typing import Any

from steamship.base import Client


@dataclass
class BlockifyRequest:
    type: str = None
    pluginInstance: str = None
    id: str = None
    handle: str = None
    name: str = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "BlockifyRequest":
        return BlockifyRequest(
            type=d.get("type", None),
            pluginInstance=d.get("pluginInstance", None),
            id=d.get("id", None),
            handle=d.get("handle", None),
            name=d.get("name", None),
        )



from steamship import Corpus, File, MimeTypes

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client
from tests.utils.random import random_name


def test_corpus_create():
    client = get_steamship_client()

    corpus = Corpus.create(client)
    assert corpus.data is not None
    corpus.data.delete()

    corpus = Corpus.create(client).data

    a = File.create(
        client, content="A", corpus_id=corpus.id, mime_type=MimeTypes.MKD
    ).data
    assert a.id is not None
    assert a.mime_type == MimeTypes.MKD
    assert a.corpus_id == corpus.id

    a = File.scrape(
        client,
        corpus_id=corpus.id,
        url="https://edwardbenson.com/2020/10/gpt3-travel-agent",
    ).data
    assert a.id is not None
    assert a.corpus_id == corpus.id

    resp = File.list(client, corpus_id=corpus.id).data
    assert len(resp.files) == 2


def test_corpus_upsert():
    client = get_steamship_client()
    corpus1 = Corpus.create(client).data
    assert corpus1.id is not None

    # This finds the same corpus as corpus1
    corpus2 = Corpus.create(client)
    assert corpus2.data is not None
    assert corpus2.error is None
    # Commenting out below, which deletes corpus1, which results in the corpus3 test failing
    # corpus2.data.delete()

    corpus2 = Corpus.create(client, handle=random_name()).data
    assert corpus1.id != corpus2.id
    assert corpus2.id is not None
    corpus2.delete()

    assert corpus1.handle is not None
    assert len(corpus1.handle) > 0
    corpus3 = Corpus.create(client, handle=corpus1.handle)
    assert corpus3.error is None
    assert corpus3.data.id is not None
    assert corpus1.id == corpus3.data.id
    corpus1.delete()


def test_corpus_delete():
    client = get_steamship_client()
    corpus1 = Corpus.create(client).data
    resp = corpus1.delete()
    assert resp.error is None
    assert resp.data is not None
    assert resp.data.id == corpus1.id

    a = File.create(client, content="A", corpus_id=corpus1.id, mime_type=MimeTypes.MKD)
    assert a.data is None
    assert a.error is not None


def test_corpus_delete_cascade():
    client = get_steamship_client()

    corpus1 = Corpus.create(client=client).data

    a = File.create(
        client=client, corpus_id=corpus1.id, content="A", mime_type=MimeTypes.MKD
    ).data

    _ = a.refresh()
    corpus1.delete()

    # Now verify the file isn't there!
    aa = File.get(client, id=a.id)
    assert aa.data is None
    assert aa.error is not None


# TODO: Add tests w/ operations in different spaces



from typing import Any, Dict, Optional


def safe_get(d: Dict, key: str, default: Any = None) -> Optional[Any]:
    """Safely a value from dictionairy using a specific key"""
    return d.get(key, default) or default



from steamship.plugin.inputs.raw_data_plugin_input import RawDataPluginInput
from steamship.plugin.service import PluginRequest
from tests.demo_apps.plugins.blockifiers.blockifier import DummyBlockifierPlugin

__copyright__ = "Steamship"
__license__ = "MIT"


def test_resp():
    blockifier = DummyBlockifierPlugin()
    request = PluginRequest(data=RawDataPluginInput(data="Hi there"))
    response = blockifier.run(request).data
    assert response.file is not None
    assert len(response.file.blocks) == 4

    # Now test as an HTTP Call



import logging
from http import HTTPStatus
from typing import Dict, Type

from steamship.app.app import App
from steamship.app.request import Request
from steamship.app.response import Response
from steamship.base import SteamshipError
from steamship.client.client import Steamship


def create_handler(app_cls: Type[App]):
    """Wrapper function for an Steamship app within an AWS Lambda function."""

    def _handler(event: Dict, _: Dict = None) -> Response:
        try:
            client = Steamship(config_dict=event.get("clientConfig"))
        except SteamshipError as se:
            logging.error(se)
            return Response.from_obj(se)
        except Exception as ex:
            logging.error(ex)
            return Response.error(
                code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message="Plugin/App handler was unable to create Steamship client.",
                exception=ex,
            )

        try:
            request = Request.from_dict(event)
        except SteamshipError as se:
            logging.error(se)
            return Response.from_obj(se)
        except Exception as ex:
            logging.error(ex)
            return Response.error(
                code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message="Plugin/App handler was unable to parse inbound request.",
                exception=ex,
            )

        try:
            app = app_cls(client=client, config=request.invocation.config)
        except SteamshipError as se:
            return Response.from_obj(se)
        except Exception as ex:
            logging.error(ex)
            return Response.error(
                code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message="Handler was unable to initialize plugin/app.",
                exception=ex,
            )

        if not app:
            return Response.error(
                code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message="Handler was unable to construct app/plugin for invocation.",
            )

        try:
            response = app(request)
            return Response.from_obj(response)
        except SteamshipError as se:
            logging.error(se)
            return Response.from_obj(se)
        except Exception as ex:
            logging.error(ex)
            app_verb = None
            app_path = None
            if request:
                if request.invocation:
                    app_path = request.invocation.appPath
                    app_verb = request.invocation.httpVerb
            return Response.error(
                code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Handler was unable to run app/plugin method {app_verb} {app_path}",
                exception=ex,
            )

    def handler(event: Dict, context: Dict = None) -> dict:
        response = _handler(event, context)
        return response.to_dict()

    return handler



class TagKind:
    """This class provides a set of `kind` constants for Tags"""

    pos = "pos"  # Part of speech tags
    dep = "dep"  # Dependency tags
    ent = "ent"  # Entity tags, such as PERSON or ORG
    doc = "doc"  # Doc-level semantics, such as H1, P, S
    text = "text"  # Text-level semantics, such as whether the span is a phone number



import json
from dataclasses import dataclass
from typing import Any, List, Optional, Dict

from steamship.base import Client, Request, Response


@dataclass
class TagQueryRequest(Request):
    tagFilterQuery: str


@dataclass
class Tag:  # TODO (enias): Make pep8 compatible
    client: Client = None
    id: str = None
    fileId: str = None
    blockId: str = None
    kind: str = None  # E.g. ner
    name: str = None  # E.g. person
    value: Dict[str, Any] = None  # JSON Metadata
    startIdx: int = None  # w/r/t block.text. None means 0 if blockId is not None
    endIdx: int = None  # w/r/t block.text. None means -1 if blockId is not None

    @dataclass
    class CreateRequest(Request):
        id: str = None
        fileId: str = None
        blockId: str = None
        kind: str = None
        name: str = None
        startIdx: int = None
        endIdx: int = None
        value: Dict[str, Any] = None
        upsert: bool = None

        # noinspection PyUnusedLocal
        @staticmethod
        def from_dict(d: Any, client: Client = None) -> "Tag.CreateRequest":
            return Tag.CreateRequest(
                id=d.get("id"),
                fileId=d.get("fileId"),
                blockId=d.get("blockId"),
                kind=d.get("kind"),
                name=d.get("name"),
                startIdx=d.get("startIdx"),
                endIdx=d.get("endIdx"),
                value=d.get("value"),
                upsert=d.get("upsert"),
            )

        def to_dict(self):
            pass

    @dataclass
    class DeleteRequest(Request):
        id: str = None
        fileId: str = None
        blockId: str = None

    @dataclass
    class ListRequest(Request):
        fileId: str = None
        blockId: str = None

    @dataclass
    class ListResponse(Request):
        tags: List["Tag"] = None

        @staticmethod
        def from_dict(d: Any, client: Client = None) -> "Optional[Tag.ListResponse]":
            if d is None:
                return None
            return Tag.ListResponse(
                tags=[
                    Tag.from_dict(x, client=client) for x in (d.get("tags", []) or [])
                ]
            )

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "Tag":
        return Tag(
            client=client,
            id=d.get("id"),
            fileId=d.get("fileId"),
            blockId=d.get("blockId"),
            kind=d.get("kind"),
            name=d.get("name"),
            startIdx=d.get("startIdx"),
            endIdx=d.get("endIdx"),
            value=d.get("value"),
        )

    def to_dict(self) -> dict:
        return dict(
            id=self.id,
            fileId=self.fileId,
            blockId=self.blockId,
            kind=self.kind,
            name=self.name,
            startIdx=self.startIdx,
            endIdx=self.endIdx,
            value=self.value,
        )

    @staticmethod
    def create(
        client: Client,
        file_id: str = None,
        block_id: str = None,
        kind: str = None,
        name: str = None,
        start_idx: int = None,
        end_idx: int = None,
        value: Any = None,
        upsert: bool = None,
        space_id: str = None,
        space_handle: str = None,
    ) -> Response["Tag"]:
        if isinstance(value, dict) or isinstance(value, list):
            value = json.dumps(value)

        req = Tag.CreateRequest(
            fileId=file_id,
            blockId=block_id,
            kind=kind,
            name=name,
            startIdx=start_idx,
            endIdx=end_idx,
            value=value,
            upsert=upsert,
        )
        return client.post(
            "tag/create", req, expect=Tag, space_id=space_id, space_handle=space_handle
        )

    @staticmethod
    def list_public(
        client: Client,
        file_id: str = None,
        block_id: str = None,
        space_id: str = None,
        space_handle: str = None,
    ) -> Response["Tag.ListResponse"]:
        return client.post(
            "tag/list",
            Tag.ListRequest(fileId=file_id, blockId=block_id),
            expect=Tag.ListResponse,
            space_id=space_id,
            space_handle=space_handle,
        )

    def delete(self) -> Response["Tag"]:
        return self.client.post(
            "tag/delete",
            Tag.DeleteRequest(id=self.id, fileId=self.fileId, blockId=self.blockId),
            expect=Tag,
        )

    @staticmethod
    def query(
        client: Client,
        tag_filter_query: str,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response["TagQueryResponse"]:

        req = TagQueryRequest(tagFilterQuery=tag_filter_query)
        res = client.post(
            "tag/query",
            payload=req,
            expect=TagQueryResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )
        return res


@dataclass
class TagQueryResponse:
    tags: List[Tag]

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "TagQueryResponse":
        return TagQueryResponse(
            tags=[Tag.from_dict(tag, client=client) for tag in d.get("tags", [])]
        )



from dataclasses import dataclass
from typing import Any, Dict, List

from steamship.base import Client


@dataclass
class EmbedRequest:
    docs: List[str]
    pluginInstance: str
    metadata: Dict = None

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "EmbedRequest":
        # TODO (enias): Review these _: Client lines
        return EmbedRequest(
            docs=d.get("docs", None),
            pluginInstance=d.get("pluginInstance", None),
            metadata=d.get("metadata", {}),
        )



import json
from dataclasses import dataclass
from typing import Any, Dict, List, Union

from steamship.base import Client, Request, Response, metadata_to_str
from steamship.data.search import Hit


@dataclass
class EmbedAndSearchRequest(Request):
    query: str
    docs: List[str]
    pluginInstance: str
    k: int = 1


# TODO: These types are not generics like the Swift QueryResult/QueryResults.
@dataclass
class QueryResult:
    value: Hit
    score: float
    index: int
    id: str

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "QueryResult":
        value = Hit.from_dict(d.get("value", {}))
        return QueryResult(
            value=value, score=d.get("score"), index=d.get("index"), id=d.get("id")
        )


@dataclass
class QueryResults(Request):
    items: List[QueryResult] = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "QueryResults":
        # TODO: Try to always use client through inheritance
        items = [QueryResult.from_dict(h) for h in (d.get("items", []) or [])]
        return QueryResults(items=items)


@dataclass
class EmbeddedItem:
    id: str = None
    indexId: str = None
    fileId: str = None
    blockId: str = None
    tagId: str = None
    value: str = None
    externalId: str = None
    externalType: str = None
    metadata: Any = None
    embedding: List[float] = None

    def clone_for_insert(self) -> "EmbeddedItem":
        """Produces a clone with a string representation of the metadata"""
        ret = EmbeddedItem(
            id=self.id,
            indexId=self.indexId,
            fileId=self.fileId,
            blockId=self.blockId,
            tagId=self.tagId,
            value=self.value,
            externalId=self.externalId,
            externalType=self.externalType,
            metadata=self.metadata,
            embedding=self.embedding,
        )
        if isinstance(ret.metadata, dict) or isinstance(ret.metadata, list):
            ret.metadata = json.dumps(ret.metadata)
        return ret

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "EmbeddedItem":
        return EmbeddedItem(
            id=d.get("id", None),
            indexId=d.get("indexId", None),
            fileId=d.get("fileId", None),
            blockId=d.get("blockId", None),
            tagId=d.get("tagId", None),
            value=d.get("value", None),
            externalId=d.get("externalId", None),
            externalType=d.get("externalType", None),
            metadata=d.get("metadata", None),
            embedding=d.get("embedding", None),
        )


@dataclass
class IndexCreateRequest(Request):
    handle: str = None
    name: str = None
    pluginInstance: str = None
    upsert: bool = True
    externalId: str = None
    externalType: str = None
    metadata: Any = None


@dataclass
class IndexInsertRequest(Request):
    indexId: str
    items: List[EmbeddedItem] = None
    value: str = None
    fileId: str = None
    blockType: str = None
    externalId: str = None
    externalType: str = None
    metadata: Any = None
    reindex: bool = True


@dataclass
class IndexItemId:
    indexId: str = None
    id: str = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "IndexItemId":
        return IndexItemId(indexId=d.get("indexId", None), id=d.get("id", None))


@dataclass
class IndexInsertResponse:
    itemIds: List[IndexItemId] = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "IndexInsertResponse":
        return IndexInsertResponse(
            itemIds=[IndexItemId.from_dict(x) for x in (d.get("itemIds", []) or [])]
        )


@dataclass
class IndexEmbedRequest(Request):
    id: str


@dataclass
class IndexEmbedResponse:
    id: str

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "IndexEmbedResponse":
        return IndexEmbedResponse(id=d.get("id", None))


@dataclass
class IndexSearchRequest(Request):
    id: str
    query: str = None
    queries: List[str] = None
    k: int = 1
    includeMetadata: bool = False


@dataclass
class IndexSnapshotRequest(Request):
    indexId: str
    # This variable is intended only to support
    # unit testing.
    windowSize: int = None


@dataclass
class IndexSnapshotResponse:
    id: str
    snapshotId: str

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "IndexSnapshotResponse":
        return IndexSnapshotResponse(
            id=d.get("id", None), snapshotId=d.get("snapshotId", None)
        )


@dataclass
class ListSnapshotsRequest(Request):
    id: str = None


@dataclass
class ListSnapshotsResponse:
    snapshots: List[IndexSnapshotResponse]

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "IndexSnapshotResponse":
        return IndexSnapshotResponse(
            snapshots=[
                IndexSnapshotResponse.from_dict(dd)
                for dd in (d.get("snapshots", []) or [])
            ]
        )


@dataclass
class ListItemsRequest(Request):
    id: str = None
    fileId: str = None
    blockId: str = None
    spanId: str = None


@dataclass
class ListItemsResponse:
    items: List[EmbeddedItem]

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "ListItemsResponse":
        return ListItemsResponse(
            items=[EmbeddedItem.from_dict(dd) for dd in (d.get("items", []) or [])]
        )


@dataclass
class DeleteSnapshotsRequest(Request):
    snapshotId: str = None


@dataclass
class DeleteSnapshotsResponse(Request):
    snapshotId: str = None


@dataclass
class DeleteEmbeddingIndexRequest(Request):
    id: str


@dataclass
class EmbeddingIndex:
    """A persistent, read-optimized index over embeddings."""

    client: Client = None
    id: str = None
    handle: str = None
    name: str = None
    plugin: str = None
    external_id: str = None
    external_type: str = None
    metadata: str = None

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "EmbeddingIndex":
        if "embeddingIndex" in d:
            d = d["embeddingIndex"]
        elif "index" in d:
            d = d["index"]
        return EmbeddingIndex(
            client=client,
            id=d.get("id"),
            handle=d.get("handle"),
            name=d.get("name"),
            plugin=d.get("plugin"),
            external_id=d.get("externalId"),
            external_type=d.get("externalType"),
            metadata=d.get("metadata"),
        )

    def insert_file(
        self,
        file_id: str,
        block_type: str = None,
        external_id: str = None,
        external_type: str = None,
        metadata: Union[int, float, bool, str, List, Dict] = None,
        reindex: bool = True,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response[IndexInsertResponse]:
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = IndexInsertRequest(
            indexId=self.id,
            fileId=file_id,
            blockType=block_type,
            externalId=external_id,
            externalType=external_type,
            metadata=metadata,
            reindex=reindex,
        )
        return self.client.post(
            "embedding-index/item/create",
            req,
            expect=IndexInsertResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def insert_many(
        self,
        items: List[Union[EmbeddedItem, str]],
        reindex: bool = True,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response[IndexInsertResponse]:
        new_items = []
        for item in items:
            if type(item) == str:
                new_items.append(EmbeddedItem(value=item))
            else:
                new_items.append(item)

        req = IndexInsertRequest(
            indexId=self.id,
            items=[item.clone_for_insert() for item in new_items],
            reindex=reindex,
        )
        return self.client.post(
            "embedding-index/item/create",
            req,
            expect=IndexInsertResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def insert(
        self,
        value: str,
        external_id: str = None,
        external_type: str = None,
        metadata: Union[int, float, bool, str, List, Dict] = None,
        reindex: bool = True,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response[IndexInsertResponse]:

        req = IndexInsertRequest(
            indexId=self.id,
            value=value,
            externalId=external_id,
            externalType=external_type,
            metadata=metadata_to_str(metadata),
            reindex=reindex,
        )
        return self.client.post(
            "embedding-index/item/create",
            req,
            expect=IndexInsertResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def embed(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> Response[IndexEmbedResponse]:
        req = IndexEmbedRequest(self.id)
        return self.client.post(
            "embedding-index/embed",
            req,
            expect=IndexEmbedResponse,
            asynchronous=True,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def create_snapshot(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> Response[IndexSnapshotResponse]:
        req = IndexSnapshotRequest(indexId=self.id)
        return self.client.post(
            "embedding-index/snapshot/create",
            req,
            expect=IndexSnapshotResponse,
            asynchronous=True,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    # TODO (enias): Can these be generic list operations for all file types?
    def list_snapshots(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> Response[ListSnapshotsResponse]:
        req = ListSnapshotsRequest(id=self.id)
        return self.client.post(
            "embedding-index/snapshot/list",
            req,
            expect=ListSnapshotsResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def list_items(
        self,
        file_id: str = None,
        block_id: str = None,
        span_id: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response[ListItemsResponse]:
        req = ListItemsRequest(
            id=self.id, fileId=file_id, blockId=block_id, spanId=span_id
        )
        return self.client.post(
            "embedding-index/item/list",
            req,
            expect=ListItemsResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def delete_snapshot(
        self,
        snapshot_id: str,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response[DeleteSnapshotsResponse]:
        req = DeleteSnapshotsRequest(snapshotId=snapshot_id)
        return self.client.post(
            "embedding-index/snapshot/delete",
            req,
            expect=DeleteSnapshotsResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def delete(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> "Response[EmbeddingIndex]":
        return self.client.post(
            "embedding-index/delete",
            DeleteEmbeddingIndexRequest(id=self.id),
            expect=EmbeddingIndex,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def search(
        self,
        query: Union[str, List[str]],
        k: int = 1,
        include_metadata: bool = False,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response[QueryResults]:
        if type(query) == list:
            req = IndexSearchRequest(
                self.id, queries=query, k=k, includeMetadata=include_metadata
            )
        else:
            req = IndexSearchRequest(
                self.id, query=query, k=k, includeMetadata=include_metadata
            )
        ret = self.client.post(
            "embedding-index/search",
            req,
            expect=QueryResults,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

        return ret


    @staticmethod
    def create(
        client: Client,
        handle: str = None,
        name: str = None,
        plugin_instance: str = None,
        upsert: bool = True,
        external_id: str = None,
        external_type: str = None,
        metadata: Any = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> "Response[EmbeddingIndex]":
        req = IndexCreateRequest(
            handle=handle,
            name=name,
            pluginInstance=plugin_instance,
            upsert=upsert,
            externalId=external_id,
            externalType=external_type,
            metadata=metadata,
        )
        return client.post(
            "embedding-index/create",
            req,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
            expect=EmbeddingIndex,
        )



from dataclasses import dataclass
from typing import Any, Dict

from steamship.base import Client, Request
from steamship.base.response import Response
from steamship.plugin.inputs.export_plugin_input import ExportPluginInput
from steamship.plugin.inputs.training_parameter_plugin_input import (
    TrainingParameterPluginInput,
)
from steamship.plugin.outputs.raw_data_plugin_output import RawDataPluginOutput
from steamship.plugin.outputs.training_parameter_plugin_output import (
    TrainingParameterPluginOutput,
)


class PluginInstance:
    pass


@dataclass
class CreatePluginInstanceRequest(Request):
    id: str = None
    pluginId: str = None
    pluginHandle: str = None
    pluginVersionId: str = None
    pluginVersionHandle: str = None
    handle: str = None
    upsert: bool = None
    config: Dict[str, Any] = None


@dataclass
class DeletePluginInstanceRequest(Request):
    id: str


@dataclass
class PluginInstance:
    client: Client = None
    id: str = None
    handle: str = None
    plugin_id: str = None
    plugin_version_id: str = None
    user_id: str = None
    config: Dict[str, Any] = None
    spaceId: str = None

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "PluginInstance":
        if "pluginInstance" in d:
            d = d["pluginInstance"]

        return PluginInstance(
            client=client,
            id=d.get("id", None),
            handle=d.get("handle", None),
            plugin_id=d.get("pluginId", None),
            plugin_version_id=d.get("pluginVersionId", None),
            config=d.get("config", None),
            user_id=d.get("userId", None),
        )

    @staticmethod
    def create(
        client: Client,
        plugin_id: str = None,
        plugin_handle: str = None,
        plugin_version_id: str = None,
        plugin_version_handle: str = None,
        handle: str = None,
        upsert: bool = None,
        config: Dict[str, Any] = None,
    ) -> Response[PluginInstance]:
        req = CreatePluginInstanceRequest(
            handle=handle,
            pluginId=plugin_id,
            pluginHandle=plugin_handle,
            pluginVersionId=plugin_version_id,
            pluginVersionHandle=plugin_version_handle,
            upsert=upsert,
            config=config,
        )

        return client.post("plugin/instance/create", payload=req, expect=PluginInstance)

    def delete(self) -> PluginInstance:
        req = DeletePluginInstanceRequest(id=self.id)
        return self.client.post(
            "plugin/instance/delete", payload=req, expect=PluginInstance
        )

    def export(self, input: ExportPluginInput) -> Response[RawDataPluginOutput]:
        input.pluginInstance = self.handle
        return self.client.post(
            "plugin/instance/export", payload=input, expect=RawDataPluginOutput
        )

    def train(self, training_request: TrainingParameterPluginInput) -> PluginInstance:
        return self.client.post(
            "plugin/instance/train", payload=training_request, expect=PluginInstance
        )

    def get_training_parameters(
        self, training_request: TrainingParameterPluginInput
    ) -> PluginInstance:
        return self.client.post(
            "plugin/instance/getTrainingParameters",
            payload=training_request,
            expect=TrainingParameterPluginOutput,
        )


@dataclass
class ListPrivatePluginInstancesRequest(Request):
    pass



from typing import Optional


class DocTag:
    """This class provides a set of Markdown-style constants to use with Tag(kind="doc")"""

    doc = "doc"
    page = "page"  # E.g. in a PDF
    region = "region"  # E.g., abstract catchall region in a document
    header = "header"
    h1 = "h1"
    h2 = "h2"
    h3 = "h3"
    h4 = "h4"
    h5 = "h5"
    line = "line"
    title = "title"
    subtitle = "subtitle"
    footer = "footer"
    paragraph = "paragraph"
    list = "list"
    list_item = "listitem"
    link = "link"
    caption = "caption"
    image = "image"
    blockquote = "blockquote"
    blockcode = "blockcode"
    unk = "unk"
    sentence = "sentence"
    token = "token"
    span = "span"
    div = "div"
    pre = "pre"
    strong = "strong"
    emph = "emph"
    underline = "underline"
    teletype = "teletype"

    @staticmethod
    def from_html_tag(tagname: Optional[str]) -> Optional[str]:
        if tagname is None:
            return None

        name = tagname.lower().strip()

        if name == 'p':
            return DocTag.paragraph
        elif name == 'h1':
            return DocTag.h1
        elif name == 'h2':
            return DocTag.h2
        elif name == 'h3':
            return DocTag.h3
        elif name == 'h4':
            return DocTag.h4
        elif name == 'h5':
            return DocTag.h5
        elif name == 'ul':
            return DocTag.list
        elif name == 'li':
            return DocTag.list_item
        elif name == 'a':
            return DocTag.link
        elif name == 'div':
            return DocTag.div
        elif name == 'img':
            return DocTag.image
        elif name == 'span':
            return DocTag.span
        elif name == 'pre':
            return DocTag.pre
        elif name == 'code':
            return DocTag.blockcode
        elif name == 'blockquote':
            return DocTag.blockquote
        elif name == 'strong':
            return DocTag.strong
        elif name == 'b':
            return DocTag.strong
        elif name == 'emph':
            return DocTag.emph
        elif name == 'i':
            return DocTag.emph
        elif name == 'u':
            return DocTag.underline
        elif name == 'tt':
            return DocTag.teletype

        return None


import contextlib
import random
import string

from steamship import EmbeddingIndex, Steamship
from steamship.extension.corpus import Corpus


def random_name() -> str:
    letters = string.digits + string.ascii_letters
    return f"test_{''.join(random.choice(letters) for _ in range(10))}"


_TEST_EMBEDDER = "test-embedder-v1"


@contextlib.contextmanager
def random_index(steamship: Steamship, plugin_instance: str) -> EmbeddingIndex:
    index = steamship.create_index(plugin_instance=plugin_instance).data
    yield index
    index.delete()  # or whatever you need to do at exit


@contextlib.contextmanager
def random_corpus(client: Steamship) -> Corpus:
    corpus = Corpus.create(client).data
    yield corpus
    corpus.delete()



from steamship.app import App, Response, create_handler, get, post


class HelloWorld(App):
    @post("greet")
    def greet(self, name: str = "Person") -> Response:
        return Response(string=f"Hello, {name}")

    @get("space")
    def space(self) -> Response:
        return Response(string=self.client.config.space_id)


handler = create_handler(HelloWorld)



from steamship import File, PluginInstance
from tests.demo_apps.plugins.importers.plugin_file_importer import TEST_DOC

from .. import APPS_PATH

__copyright__ = "Steamship"
__license__ = "MIT"

from ..utils.client import get_steamship_client
from ..utils.deployables import deploy_plugin
from ..utils.random import random_corpus


def test_e2e_corpus_importer():
    client = get_steamship_client()
    corpus_importer_path = (
        APPS_PATH / "plugins" / "importers" / "plugin_corpus_importer.py"
    )

    test_file_importer_instance = PluginInstance.create(
        client, plugin_handle="test-fileImporter-valueOrData", upsert=True
    ).data
    with deploy_plugin(client, corpus_importer_path, "corpusImporter") as (
        plugin,
        version,
        instance,
    ):
        # with PluginVersion.(client, "corpusImporter", "do_import", app, instance) as plugin:
        with random_corpus(client) as corpus:
            resp = corpus.do_import(
                plugin_instance=instance.handle,
                file_importer_plugin_instance=test_file_importer_instance.handle,
                value="dummy-value",
            )
            resp.wait()

            # We should now have two files!
            files = File.list(client, corpus_id=corpus.id).data
            assert files.files is not None
            assert len(files.files) == 2

            for file in files.files:
                data = file.raw().data
                assert data.decode("utf-8") == TEST_DOC
                file.delete()



import json
from dataclasses import dataclass
from typing import Any, List

from steamship.base import Client, Request, Response, str_to_metadata
from steamship.base.request import IdentifierRequest


@dataclass
class Corpus:
    """A corpus of files."""

    client: Client
    id: str = None
    handle: str = None
    description: str = None
    externalId: str = None
    externalType: str = None
    metadata: Any = None

    @dataclass
    class CreateRequest(Request):
        corpusId: str = None
        handle: str = None
        description: str = None
        externalId: str = None
        externalType: str = None
        isPublic: bool = None
        metadata: str = None
        upsert: bool = None

    @dataclass
    class DeleteRequest(Request):
        corpusId: str

    @dataclass
    class ListRequest(Request):
        pass

    @dataclass
    class ListResponse(Request):
        corpora: List["Corpus"]

        # noinspection PyUnusedLocal
        @staticmethod
        def from_dict(d: Any, client: Client = None) -> "Corpus.ListResponse":
            return Corpus.ListResponse(
                corpora=[Corpus.from_dict(x) for x in (d.get("corpus", []) or [])]
            )

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "Corpus":
        if "corpus" in d:
            d = d["corpus"]

        return Corpus(
            client=client,
            id=d.get("id", None),
            handle=d.get("handle", None),
            description=d.get("description", None),
            externalId=d.get("externalId", None),
            externalType=d.get("externalType", None),
            metadata=str_to_metadata(d.get("metadata", None)),
        )

    @staticmethod
    def default(
        client: Client,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> "Response[Corpus]":  # TODO (Enias): Review
        req = IdentifierRequest()
        return client.post(
            "corpus/get",
            req,
            expect=Corpus,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    @staticmethod
    def create(
        client: Client,
        handle: str = None,
        description: str = None,
        external_id: str = None,
        external_type: str = None,
        metadata: Any = None,
        is_public: bool = False,
        upsert: bool = True,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> "Response[Corpus]":
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = Corpus.CreateRequest(
            handle=handle,
            description=description,
            isPublic=is_public,
            upsert=upsert,
            externalId=external_id,
            externalType=external_type,
            metadata=metadata,
        )
        return client.post(
            "corpus/create",
            req,
            expect=Corpus,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def delete(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> Response["Corpus"]:
        return self.client.post(
            "corpus/delete",
            IdentifierRequest(id=self.id),
            expect=Corpus,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )



# import contextlib
# import time
# from pathlib import Path
# from typing import Dict
#
# from steamship import App, AppVersion, AppInstance, Steamship
# from steamship.data.user import User
#
#
# @contextlib.contextmanager
# def deploy_app(
#         client: Steamship,
#         py_path: Path,
#         version_config_template: Dict[str, Any] = None,
#         instance_config: Dict[str, Any] = None,
# ):
#     app = App.create(client)
#     assert app.error is None
#     assert app.data is not None
#     app = app.data
#
#     zip_bytes = zip_app(py_name)
#     version = AppVersion.create(
#         client,
#         app_id=app.id,
#         filebytes=zip_bytes,
#         config_template=version_config_template,
#     )
#     # TODO: This is due to having to wait for the lambda to finish deploying.
#     # TODO: We should update the task system to allow its .wait() to depend on this.
#     time.sleep(15)
#     version.wait()
#     assert version.error is None
#     assert version.data is not None
#     version = version.data
#
#     instance = AppInstance.create(
#         client, app_id=app.id, app_version_id=version.id, config=instance_config
#     )
#     instance.wait()
#     assert instance.error is None
#     assert instance.data is not None
#     instance = instance.data
#
#     assert instance.app_id == app.id
#     assert instance.app_version_id == version.id
#
#     user = User.current(client).data
#
#     assert instance.user_handle == user.handle
#     assert instance.user_id == user.id
#
#     yield app, version, instance
#
#     res = instance.delete()
#     assert res.error is None
#
#     res = version.delete()
#     assert res.error is None
#
#     res = app.delete()
#     assert res.error is None



from steamship import Block, File, MimeTypes
from steamship.data.tags import Tag

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_file_tag():
    steamship = get_steamship_client()
    a = steamship.upload(content="A", mime_type=MimeTypes.MKD).data
    assert a.id is not None
    assert a.mime_type == MimeTypes.MKD

    _ = Tag.create(steamship, file_id=a.id, name="test1").data
    _ = Tag.create(steamship, file_id=a.id, name="test2").data

    tags = Tag.list_public(steamship, file_id=a.id)
    assert tags.data is not None
    assert tags.data.tags is not None
    assert len(tags.data.tags) == 2

    must = ["test1", "test2"]
    for tag in tags.data.tags:
        assert tag.name in must
        must.remove(tag.name)
    assert len(must) == 0

    for tag in tags.data.tags:
        if tag.name == "test1":
            tag.delete()

    tags = Tag.list_public(steamship, file_id=a.id)
    assert tags.data is not None
    assert tags.data.tags is not None
    assert len(tags.data.tags) == 1

    must = ["test2"]
    for tag in tags.data.tags:
        assert tag.name in must
        must.remove(tag.name)
    assert len(must) == 0

    tags.data.tags[0].delete()

    tags = Tag.list_public(steamship, file_id=a.id)
    assert tags.data is not None
    assert tags.data.tags is not None
    assert len(tags.data.tags) == 0

    a.delete()


def test_query():
    client = get_steamship_client()
    a = File.create(
        client=client,
        blocks=[
            Block.CreateRequest(text="A", tags=[Tag.CreateRequest(name="BlockTag")]),
            Block.CreateRequest(text="B"),
        ],
    ).data
    assert a.id is not None
    a = a.refresh().data
    b = File.create(
        client=client,
        blocks=[
            Block.CreateRequest(text="A"),
            Block.CreateRequest(text="B", tags=[Tag.CreateRequest(name="Test")]),
        ],
        tags=[Tag.CreateRequest(name="FileTag")],
    ).data
    assert b.id is not None
    b = b.refresh().data

    tags = Tag.query(
        client=client, tag_filter_query='blocktag and name "BlockTag"'
    ).data.tags
    assert len(tags) == 1
    assert tags[0].id == a.blocks[0].tags[0].id

    tags = Tag.query(
        client=client, tag_filter_query='blocktag and name "Test"'
    ).data.tags
    assert len(tags) == 1
    assert tags[0].id == b.blocks[1].tags[0].id

    a.delete()
    b.delete()



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Any, Callable, Generic, TypeVar, Union

from steamship.app.response import Response
from steamship.base import Client
from steamship.base.response import SteamshipError

# Note!
# =====
#
# This the files in this package are for Plugin Implementors.
# If you are using the Steamship Client, you probably are looking for either steamship.client or steamship.data
#

T = TypeVar("T")
U = TypeVar("U")


@dataclass
class PluginRequest(Generic[T]):
    data: T = None

    @staticmethod
    def from_dict(
        d: Any,
        subclass_request_from_dict: Callable[[dict, Client], dict] = None,
        client: Client = None,
    ) -> "PluginRequest[T]":
        data = None
        if "data" in d:
            if subclass_request_from_dict is not None:
                data = subclass_request_from_dict(d["data"], client)
            else:
                raise SteamshipError(
                    message="No `subclass_request_from_dict` provided to parse inbound dict request."
                )
        return PluginRequest(data=data)

    def to_dict(self) -> dict:
        if self.data is None:
            return {}
        else:
            return {"data": self.data.to_dict()}


class PluginService(ABC, Generic[T, U]):
    @abstractmethod
    def run(self, request: PluginRequest[T]) -> Union[U, Response[U]]:
        pass

    @classmethod
    @abstractmethod
    def subclass_request_from_dict(
        cls, d: Any, client: Client = None
    ) -> PluginRequest[T]:
        pass

    @classmethod
    def parse_request(cls, request: Union[PluginRequest[T], dict]) -> PluginRequest[T]:
        if type(request) == dict:
            try:
                request = PluginRequest[T].from_dict(
                    request, subclass_request_from_dict=cls.subclass_request_from_dict
                )
            except Exception as error:
                raise SteamshipError(
                    message="Unable to parse input request.", error=error
                )
        return request

    @classmethod
    def response_to_dict(
        cls, response: Union[SteamshipError, Response[U], U, dict]
    ) -> Response[U]:
        try:
            if type(response) == Response:
                return response
            elif type(response) == SteamshipError:
                return Response(error=response)
            else:
                return Response(data=response)
        except SteamshipError as remote_error:
            return Response[U](error=remote_error)
        except Exception as error:
            return Response[U](
                error=SteamshipError(
                    message="Unhandled exception completing your request", error=error
                )
            )



from steamship import AppInstance

__copyright__ = "Steamship"
__license__ = "MIT"

from tests import APPS_PATH
from tests.utils.client import get_steamship_client
from tests.utils.deployables import deploy_app


def test_configurable_instance_invoke():
    greeting1 = "Hola"
    config_template = {"greeting": {"type": "string"}}
    instance_config = {"greeting": greeting1}
    client = get_steamship_client()
    hello_world_path = APPS_PATH / "apps" / "configurable_hello_world.py"

    with deploy_app(
        client,
        hello_world_path,
        version_config_template=config_template,
        instance_config=instance_config,
    ) as (app, version, instance):
        # Now let's invoke it!
        # Note: we're invoking the data at configurable_hello_world.py in the tests/demo_apps folder
        res = instance.post("greet").data
        assert res == f"{greeting1}, Person"

        greeting2 = "Hallo"
        instance_config2 = {"greeting": greeting2}
        instance2 = AppInstance.create(
            client, app_id=app.id, app_version_id=version.id, config=instance_config2
        )
        instance2.wait()
        assert instance2.error is None
        assert instance2.data is not None
        instance2 = instance2.data

        res2 = instance2.post("greet").data
        assert res2 == f"{greeting2}, Person"



from steamship import App, AppVersion

__copyright__ = "Steamship"
__license__ = "MIT"

from tests import APPS_PATH
from tests.utils.client import get_steamship_client
from tests.utils.deployables import zip_deployable


def test_version_create():
    client = get_steamship_client()
    demo_app_path = APPS_PATH / "apps" / "demo_app.py"

    app = App.create(client).data
    zip_bytes = zip_deployable(demo_app_path)

    version = AppVersion.create(client, app_id=app.id, filebytes=zip_bytes)

    version.wait()

    res = version.data.delete()
    assert res.error is None

    res = app.delete()
    assert res.error is None



# Plugin
#
# This file contains the abstractions for managing Steamship plugins.
# To see how to implement a Steamship Plugin, see service.py in the same folder.
#
#

import json
from dataclasses import dataclass
from typing import Any, Dict, List, Union, Optional

from steamship.base.client import Client
from steamship.base.request import Request
from steamship.base.response import Response


class Plugin:
    pass


class TrainingPlatform:
    custom = "lambda"
    managed = "ecs"


@dataclass
class CreatePluginRequest(Request):
    trainingPlatform: str
    id: str = None
    type: str = None
    transport: str = None
    isPublic: bool = None
    handle: str = None
    description: str = None
    metadata: str = None
    upsert: bool = None


@dataclass
class DeletePluginRequest(Request):
    id: str


@dataclass
class ListPublicPluginsRequest(Request):
    type: str


@dataclass
class ListPrivatePluginsRequest(Request):
    type: str


@dataclass
class ListPluginsResponse(Request):
    plugins: List[Plugin]

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "ListPluginsResponse":
        return ListPluginsResponse(
            plugins=[
                Plugin.from_dict(x, client=client) for x in (d.get("plugins", []) or [])
            ]
        )


@dataclass
class GetPluginRequest(Request):
    type: str = None
    id: str = None
    handle: str = None


class PluginType:
    parser = "parser"
    classifier = "classifier"
    tagger = "tagger"
    embedder = "embedder"


class PluginAdapterType:
    steamshipDocker = "steamshipDocker"
    steamshipSagemaker = "steamshipSagemaker"
    huggingface = "huggingface"
    openai = "openai"


class PluginTargetType:
    file = "file"
    corpus = "corpus"
    space = "space"


class LimitUnit:
    words = "words"
    characters = "characters"
    bytes = "bytes"


@dataclass
class Plugin:
    client: Client = None
    id: str = None
    type: str = None
    transport: str = None
    isPublic: bool = None
    trainingPlatform: str = None
    handle: str = None
    description: str = None
    metadata: str = None

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "Plugin":
        if "plugin" in d:
            d = d["plugin"]

        return Plugin(
            client=client,
            id=d.get("id", None),
            type=d.get("type", None),
            transport=d.get("transport", None),
            isPublic=d.get("isPublic", None),
            trainingPlatform=d.get("trainingPlatform", None),
            handle=d.get("handle", None),
            description=d.get("description", None),
            metadata=d.get("metadata", None),
        )

    @staticmethod
    def create(
        client: Client,
        training_platform: Optional[str],
        description: str,
        type_: str,
        transport: str,
        is_public: bool,
        handle: str = None,
        metadata: Union[str, Dict, List] = None,
        upsert: bool = None,
        space_id: str = None,
        space_handle: str = None,
    ) -> Response[Plugin]:
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = CreatePluginRequest(
            trainingPlatform=training_platform,
            type=type_,
            transport=transport,
            isPublic=is_public,
            handle=handle,
            description=description,
            metadata=metadata,
            upsert=upsert,
        )
        return client.post(
            "plugin/create",
            req,
            expect=Plugin,
            space_id=space_id,
            space_handle=space_handle,
        )

    @staticmethod
    def list(
        client: Client, t: str = None, space_id: str = None, space_handle: str = None
    ) -> Response[ListPluginsResponse]:
        return client.post(
            "plugin/list",
            ListPublicPluginsRequest(type=t),
            expect=ListPluginsResponse,
            space_id=space_id,
            space_handle=space_handle,
        )

    @staticmethod
    def get(client: Client, handle: str):
        return client.post("plugin/get", GetPluginRequest(handle=handle), expect=Plugin)

    def delete(self) -> Response[Plugin]:
        return self.client.post(
            "plugin/delete",
            DeletePluginRequest(id=self.id),
            expect=Plugin,
        )



from typing import List

from steamship import Block, File, Tag, TagKind, TextTag
from steamship.app import App, Response, create_handler, post
from steamship.plugin.embedder import Embedder
from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginRequest

FEATURES = [
    "employee",
    "roses",
    "run",
    "bike",
    "ted",
    "grace",
    "violets",
    "sugar",
    "sweet",
    "cake",
    "flour",
    "chocolate",
    "vanilla",
    "flavors",
    "flavor",
    "can",
    "armadillo",
    "pizza",
    "ship",
    "run",
    "jerry",
    "ronaldo",
    "ted",
    "brenda",
    "susan",
    "sam",
    "jerry",
    "shoe",
    "cheese",
    "water",
    "drink",
    "glass",
    "egg",
    "sleep",
    "code",
    "jonathan",
    "dolphin",
    "apple",
    "orange",
    "number",
    "0",
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
    "10",
    "11",
    "12",
    "13",
    "14",
    "15",
    "16",
    "17",
    "bad",
    "terrible",
    "average",
    "good",
]

DIMENSIONALITY = len(FEATURES)


def embed(s: str) -> List[float]:
    s = s.lower()
    return list(map(lambda word: 1.0 if word in s else 0.0, FEATURES))


def _embed_to_tag(s: str) -> Tag.CreateRequest:
    embedding = embed(s)
    return Tag.CreateRequest(
        kind=TagKind.text, name=TextTag.embedding, value={TextTag.embedding: embedding}
    )


def _embed_block(block: Block) -> Block.CreateRequest:
    return Block.CreateRequest(
        id=block.id, text=block.text, tags=[_embed_to_tag(block.text)]
    )


class TestEmbedderPlugin(Embedder, App):
    def run(
        self, request: PluginRequest[BlockAndTagPluginInput]
    ) -> Response[BlockAndTagPluginOutput]:
        updated_blocks = [_embed_block(block) for block in request.data.file.blocks]
        return Response(
            data=BlockAndTagPluginOutput(file=File.CreateRequest(blocks=updated_blocks))
        )

    @post("tag")
    def embed(self, **kwargs) -> Response:
        embed_request = Embedder.parse_request(request=kwargs)
        return self.run(embed_request)


handler = create_handler(TestEmbedderPlugin)



import re

from steamship import Block, DocTag, Tag
from steamship.app import App, Response, create_handler, post
from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginRequest
from steamship.plugin.tagger import Tagger


def tag_sentences(block: Block):
    """Splits the document into sentences by assuming a period is a sentence divider."""
    # Add the period back
    tags = []
    for m in re.finditer(r"[^.]+", block.text):
        tags.append(
            Tag(
                kind=DocTag.doc,
                name=DocTag.sentence,
                startIdx=m.start(),
                endIdx=m.end() + 1,
            )
        )
    if block.tags:
        block.tags.extend(tags)
    else:
        block.tags = tags


def _make_test_response(request: BlockAndTagPluginInput) -> BlockAndTagPluginOutput:
    for block in request.file.blocks:
        tag_sentences(block)
    response = BlockAndTagPluginOutput(request.file)
    return response


class TestParserPlugin(Tagger, App):
    # TODO: WARNING! We will need to implement some logic that prevents
    # a distributed endless loop. E.g., a parser plugin returning the results
    # of using the Steamship client to call parse.. via itself!

    def run(
        self, request: PluginRequest[BlockAndTagPluginInput]
    ) -> Response[BlockAndTagPluginOutput]:
        if request.data is not None:
            return Response(data=_make_test_response(request.data))

    @post("tag")
    def parse(self, **kwargs) -> dict:
        parse_request = Tagger.parse_request(request=kwargs)
        return self.run(parse_request)


handler = create_handler(TestParserPlugin)



from dataclasses import asdict, dataclass
from typing import Any

from steamship import File
from steamship.base import Client, Request


@dataclass
class TagRequest(Request):
    type: str = None
    id: str = None
    name: str = None
    handle: str = None
    pluginInstance: str = None
    file: File.CreateRequest = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "TagRequest":

        return TagRequest(
            type=d.get("type", None),
            id=d.get("id", None),
            name=d.get("name", None),
            handle=d.get("handle", None),
            pluginInstance=d.get("pluginInstance", None),
            file=File.CreateRequest.from_dict(d.get("file", {})),
        )

    def to_dict(self) -> dict:
        return asdict(self)


@dataclass
class TagResponse:
    file: File = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "TagResponse":
        return TagResponse(file=File.from_dict(d.get("file", {})))

    def to_dict(self) -> dict:
        return asdict(self)



from steamship.app import Response
from steamship.data.file import File
from steamship.plugin.outputs.raw_data_plugin_output import RawDataPluginOutput
from steamship.plugin.service import PluginRequest

__copyright__ = "Steamship"
__license__ = "MIT"

import base64

from tests.demo_apps.plugins.importers.plugin_file_importer import (
    TEST_DOC,
    TestFileImporterPlugin,
)

TEST_REQ = File.CreateRequest(value="Hi there.")
TEST_PLUGIN_REQ = PluginRequest(data=TEST_REQ)
TEST_REQ_DICT = TEST_PLUGIN_REQ.to_dict()


def _test_resp(res):
    assert type(res) == Response
    assert type(res.data) == RawDataPluginOutput
    b64 = base64.b64encode(TEST_DOC.encode("utf-8")).decode("utf-8")
    assert res.data.data == b64


def test_importer():
    importer = TestFileImporterPlugin()
    res = importer.run(TEST_PLUGIN_REQ)
    _test_resp(res)



from steamship.app import App, Response, create_handler, post


class HelloWorld(App):
    @post("greet")
    def greet(self, name: str = "Person") -> Response:
        greeting = self.config["greeting"]
        return Response(string=f"{greeting}, {name}")


handler = create_handler(HelloWorld)



from steamship import Block, File, Tag
from steamship.app import App, Response, create_handler, post
from steamship.data.tags import DocTag, TagKind
from steamship.plugin.blockifier import Blockifier
from steamship.plugin.inputs.raw_data_plugin_input import RawDataPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginRequest

# Note 1: this aligns with the same document in the internal Engine test.
# Note 2: This should be duplicated from the test_importer because of the way our test system will
#         bundle this into an app deployment package (importing won't work!)
HANDLE = "test-blockifier-plugin-v1"
TEST_H1 = "A Poem"
TEST_S1 = "Roses are red."
TEST_S2 = "Violets are blue."
TEST_S3 = "Sugar is sweet, and I love you."
TEST_DOC = f"# {TEST_H1}\n\n{TEST_S1} {TEST_S2}\n\n{TEST_S3}\n"


class DummyBlockifierPlugin(Blockifier, App):
    def run(
        self, request: PluginRequest[RawDataPluginInput]
    ) -> Response[BlockAndTagPluginOutput]:
        return Response(
            data=BlockAndTagPluginOutput(
                file=File.CreateRequest(
                    blocks=[
                        Block.CreateRequest(
                            text=TEST_H1,
                            tags=[Tag.CreateRequest(kind=TagKind.doc, name=DocTag.h1)],
                        ),
                        Block.CreateRequest(
                            text=TEST_S1,
                            tags=[
                                Tag.CreateRequest(
                                    kind=TagKind.doc, name=DocTag.sentence
                                )
                            ],
                        ),
                        Block.CreateRequest(
                            text=TEST_S2,
                            tags=[
                                Tag.CreateRequest(
                                    kind=TagKind.doc, name=DocTag.sentence
                                )
                            ],
                        ),
                        Block.CreateRequest(
                            text=TEST_S3,
                            tags=[
                                Tag.CreateRequest(
                                    kind=TagKind.doc, name=DocTag.paragraph
                                )
                            ],
                        ),
                    ]
                )
            )
        )

    @post("blockify")
    def blockify(self, **kwargs) -> Response:
        raw_data_plugin_input = Blockifier.parse_request(request=kwargs)
        return self.run(raw_data_plugin_input)


handler = create_handler(DummyBlockifierPlugin)



from dataclasses import dataclass
from typing import Any, Dict, Optional

from steamship import File
from steamship.base import Client


@dataclass
class BlockAndTagPluginInput:
    file: File = None

    @staticmethod
    def from_dict(
        d: Any = None, client: Client = None
    ) -> "Optional[BlockAndTagPluginInput]":
        if d is None:
            return None

        return BlockAndTagPluginInput(
            file=File.from_dict(d.get("file", None), client=client)
        )

    def to_dict(self) -> Dict:
        if self.file is None:
            return {}
        return dict(file=self.file.to_dict())



"""CSV Blockifier - Steamship Plugin."""

import csv
import io
from typing import Dict, List, Optional, Union

from pydantic import BaseModel, constr

from steamship.app import App, Response, create_handler, post
from steamship.base.error import SteamshipError
from steamship.data.block import Block
from steamship.data.file import File
from steamship.data.tags import Tag
from steamship.plugin.blockifier import Blockifier
from steamship.plugin.inputs.raw_data_plugin_input import RawDataPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginRequest


class Config(BaseModel):
    def __init__(self, **kwargs):
        kwargs = {k: v for k, v in kwargs.items() if v}
        super().__init__(**kwargs)


class CsvBlockifierConfig(Config):
    text_column: str
    tag_columns: List[str]
    delimiter: Optional[str] = ","
    quotechar: Optional[constr(max_length=1)] = '"'
    escapechar: Optional[constr(max_length=1)] = "\\"
    newline: Optional[str] = "\\n"
    skipinitialspace: Optional[bool] = False
    tag_kind: Optional[str] = None

    def __init__(self, **kwargs):
        if isinstance(kwargs["tag_columns"], str):
            kwargs["tag_columns"] = kwargs["tag_columns"].split(",")
        super().__init__(**kwargs)


class CsvBlockifier(Blockifier, App):
    """Converts CSV or TSV into Tagged Steamship Blocks.

    A CSV file is mapped onto 1 File
    Each row in the CSV file is mapped onto a block
    Each block has text which is extracted from a single column
    Each block has one or more tags extracted from the values in one ore more columns
    """

    # noinspection PyUnusedLocal
    def __init__(self, client=None, config: Dict[str, any] = None):
        super().__init__()
        self.config = CsvBlockifierConfig(**config)

    def _get_text(self, row) -> str:
        text = row.get(self.config.text_column)
        if text:
            text = text.replace(self.config.newline, "\n")
        return text

    def _get_tags(self, row) -> List[str]:
        tag_values = []
        for tag_column in self.config.tag_columns:
            tag_value = row.get(tag_column)
            if tag_value:
                tag_values.append(tag_value.replace(self.config.newline, "\n"))
        return tag_values

    def run(
        self, request: PluginRequest[RawDataPluginInput]
    ) -> Union[Response, Response[BlockAndTagPluginOutput]]:
        if request is None or request.data is None or request.data.data is None:
            return Response(
                error=SteamshipError(
                    message="Missing data field on the incoming request."
                )
            )
        data = request.data.data  # TODO (enias): Simplify
        if isinstance(data, bytes):
            data = data.decode("utf-8")

        if type(data) != str:
            return Response(
                error=SteamshipError(
                    message="The incoming data was not of expected String type"
                )
            )

        reader = csv.DictReader(
            io.StringIO(data),
            delimiter=self.config.delimiter,
            quotechar=self.config.quotechar,
            escapechar=self.config.escapechar,
            skipinitialspace=self.config.skipinitialspace,
        )
        file = File(blocks=[])
        for row in reader:
            text = self._get_text(row)
            tag_values = self._get_tags(row)

            block = Block.CreateRequest(
                text=text,
                tags=[
                    Tag.CreateRequest(kind=self.config.tag_kind, name=tag_value)
                    for tag_value in tag_values
                ],
            )
            file.blocks.append(block)

        return Response(data=BlockAndTagPluginOutput(file=file))

    @post("blockify")
    def blockify(self, **kwargs) -> Response:
        """App endpoint for our plugin.

        The `run` method above implements the Plugin interface for a Converter.
        This `convert` method exposes it over an HTTP endpoint as a Steamship App.

        When developing your own plugin, you can almost always leave the below code unchanged.
        """
        blockify_request = Blockifier.parse_request(request=kwargs)
        return self.run(blockify_request)


handler = create_handler(CsvBlockifier)



# noinspection PyPackageRequirements
import pytest

from steamship.data.plugin import Plugin, PluginAdapterType, PluginType

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_plugin_create():
    steamship = get_steamship_client()

    my_plugins = Plugin.list(steamship).data
    orig_count = len(my_plugins.plugins)

    plugin_args = dict(
        name="name",
        client=steamship,
        description="This is just for test",
        type_=PluginType.embedder,
        transport=PluginAdapterType.steamshipDocker,
        is_public=True,
    )

    required_fields = {"name", "description", "type_"}
    for required_field in required_fields:
        with pytest.raises(Exception):
            del plugin_args[required_field]
            # noinspection PyArgumentList
            _ = Plugin.create(**plugin_args)

    my_plugins = Plugin.list(steamship).data
    assert len(my_plugins.plugins) == orig_count

    plugin = Plugin.create(
        client=steamship,
        training_platform=None,
        description="This is just for test",
        type_=PluginType.tagger,
        transport=PluginAdapterType.steamshipDocker,
        is_public=False,
    ).data
    my_plugins = Plugin.list(steamship).data
    assert len(my_plugins.plugins) == orig_count + 1

    # No upsert doesn't work
    plugin_x = Plugin.create(
        client=steamship,
        training_platform=None,
        handle=plugin.handle,
        description="This is just for test",
        type_=PluginType.tagger,
        transport=PluginAdapterType.steamshipDocker,
        is_public=False,
    )
    assert plugin_x.error is not None
    assert plugin_x.data is None

    # Upsert does work
    plugin2 = Plugin.create(
        client=steamship,
        training_platform=None,
        handle=plugin.handle,
        description="This is just for test 2",
        type_=PluginType.tagger,
        transport=PluginAdapterType.steamshipDocker,
        is_public=False,
        upsert=True,
    ).data

    assert plugin2.id == plugin.id

    my_plugins = Plugin.list(steamship).data
    assert len(my_plugins.plugins) == orig_count + 1

    assert plugin2.id in [plugin.id for plugin in my_plugins.plugins]
    assert plugin2.description in [plugin.description for plugin in my_plugins.plugins]

    # assert(my_plugins.plugins[0].description != plugin.description)

    plugin.delete()

    my_plugins = Plugin.list(steamship).data
    assert len(my_plugins.plugins) == orig_count


def test_plugin_public():
    steamship = get_steamship_client()

    resp = Plugin.list(steamship).data
    assert resp.plugins is not None
    plugins = resp.plugins

    assert len(plugins) > 0

    # Make sure they can't be deleted.
    res = plugins[0].delete()
    assert res.error is not None
    assert res.data is None



from dataclasses import dataclass
from typing import Any, Dict, Optional

from steamship.base import Client


@dataclass
class ExportPluginInput:
    pluginInstance: str = None
    id: str = None
    handle: str = None
    type: str = None
    filename: str = None
    query: str = None

    # noinspection PyUnusedLocal
    @staticmethod
    def from_dict(
        d: Any = None, client: Client = None
    ) -> "Optional[ExportPluginInput]":
        if d is None:
            return None

        return ExportPluginInput(
            pluginInstance=d.get("pluginInstance", None),
            id=d.get("id", None),
            handle=d.get("id", None),
            type=d.get("type", None),
            filename=d.get("filename", None),
            query=d.get("query", None),
        )

    def to_dict(self) -> Dict:
        return dict(
            pluginInstance=self.pluginInstance,
            id=self.id,
            handle=self.handle,
            type=self.type,
            filename=self.filename,
            query=self.query,
        )



from steamship import Space

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_default_space():
    client = get_steamship_client()
    space = Space.get(client=client).data
    assert space is not None
    assert space.handle == "default"


def test_delete_space():
    client = get_steamship_client()
    default = Space.get(client=client).data
    space1 = Space.create(client=client, handle="test").data
    assert default.id is not None
    assert space1.id is not None
    assert default.id != space1.id

    space1.delete()
    space1a = Space.get(client=client, space_id=space1.id)
    assert space1a.data is None
    assert space1a.error is not None

    space1 = Space.create(client=client, handle="test").data
    assert default.id is not None
    assert space1.id is not None
    assert default.id != space1.id

    space1a = Space.get(client=client, space_id=space1.id)
    assert space1a.data is not None
    assert space1a.error is None

    space1.delete()
    space1a = Space.get(client=client, space_id=space1.id)
    assert space1a.data is None
    assert space1a.error is not None


def test_get_space():
    client = get_steamship_client()
    default = Space.get(client=client).data
    space1 = Space.create(client=client, handle="test").data
    space1a = Space.get(client=client, space_id=space1.id).data
    assert space1a.id == space1.id
    assert space1a.id != default.id
    assert space1a.handle == space1.handle


def test_create_use_delete_space():
    client = get_steamship_client()
    default = Space.get(client=client).data
    space1 = Space.create(client=client, handle="test").data
    space2 = Space.create(client=client, handle="test2").data

    assert space1 is not None
    assert space1.handle == "test"

    assert space2 is not None
    assert space2.handle == "test2"

    assert space2.id != space1.id
    assert space1.id != default.id
    assert space2.id != default.id

    space1a = Space.get(client=client, space_id=space1.id).data
    space1b = Space.get(client=client, space_handle=space1.handle).data
    space1c = Space.get(client=client, space=space1).data

    assert space1.id == space1a.id
    assert space1.id == space1b.id
    assert space1.id == space1c.id

    space1ad = space1a.delete()
    assert space1ad.data is not None
    assert space1ad.error is None

    # These two are the same space! You can't delete twice!
    space1bd = space1b.delete()
    assert space1bd.data is None
    assert space1bd.error is not None
    space1cd = space1c.delete()
    assert space1cd.data is None
    assert space1cd.error is not None

    space2.delete()

    space1a_deleted = Space.get(client=client, space_id=space1.id)
    assert space1a_deleted.data is None
    assert space1a_deleted.error is not None



from steamship.base.response import TaskState

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_file_upload_then_parse():
    steamship = get_steamship_client()

    a = steamship.upload(content="This is a test.").data
    assert a.id is not None

    q1 = a.refresh().data
    assert len(q1.blocks) == 0

    task = a.blockify(plugin_instance="markdown-blockifier-default-1.0")
    assert task.error is None
    assert task.task is not None
    assert task.task.state == TaskState.waiting

    task.wait()
    assert task.error is None
    assert task.task is not None
    assert task.task.state == TaskState.succeeded

    q1 = a.refresh().data
    assert len(q1.blocks) == 1
    assert q1.blocks[0].text == "This is a test."

    b = steamship.upload(
        content="""# Header

This is a test."""
    ).data
    assert b.id is not None

    q1 = b.refresh().data
    assert len(q1.blocks) == 0

    task = b.blockify(plugin_instance="markdown-blockifier-default-1.0")
    assert task.error is None
    assert task.task is not None
    task.wait()

    q1 = b.refresh().data
    assert len(q1.blocks) == 2
    assert q1.blocks[1].text == "This is a test."
    assert q1.blocks[0].text == "Header"

    a.delete()
    b.delete()



import json
from typing import Dict, List, Optional, Union

Metadata = Union[int, float, bool, str, List, Dict]


def str_to_metadata(s: str) -> Optional[Metadata]:
    if s is None:
        return None
    return json.loads(s)


def metadata_to_str(m: Metadata) -> Optional[str]:
    if m is None:
        return None
    return json.dumps(m)



from steamship.data.block import Block
from steamship.data.file import File
from steamship.data.tags.tag import Tag

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client


def test_query():
    client = get_steamship_client()
    a = File.create(
        client=client,
        blocks=[
            Block.CreateRequest(text="A", tags=[Tag.CreateRequest(name="BlockTag")]),
            Block.CreateRequest(text="B"),
        ],
    ).data
    assert a.id is not None
    a = a.refresh().data
    b = File.create(
        client=client,
        blocks=[
            Block.CreateRequest(text="A"),
            Block.CreateRequest(text="B", tags=[Tag.CreateRequest(name="Test")]),
        ],
        tags=[Tag.CreateRequest(name="FileTag")],
    ).data
    assert b.id is not None
    b = b.refresh().data

    blocks = Block.query(
        client=client, tag_filter_query='blocktag and name "BlockTag"'
    ).data.blocks
    assert len(blocks) == 1
    assert blocks[0].id == a.blocks[0].id

    blocks = Block.query(
        client=client, tag_filter_query='blocktag and name "Test"'
    ).data.blocks
    assert len(blocks) == 1
    assert blocks[0].id == b.blocks[1].id

    a.delete()
    b.delete()



from .. import APPS_PATH

__copyright__ = "Steamship"
__license__ = "MIT"

from ..utils.client import get_steamship_client
from ..utils.deployables import deploy_plugin
from ..utils.file import upload_file


def test_e2e_tsv_blockifier_plugin():
    csv_blockifier_plugin_path = (
        APPS_PATH / "plugins" / "blockifiers" / "tsv_blockifier.py"
    )
    client = get_steamship_client()

    version_config_template = dict(
        text_column=dict(type="string"),
        tag_columns=dict(type="string"),
        tag_kind=dict(type="string"),
    )
    instance_config = dict(  # Has to match up
        text_column="Message",
        tag_columns="Category",
        tag_kind="Intent",
    )
    with deploy_plugin(
        client,
        csv_blockifier_plugin_path,
        "blockifier",
        version_config_template=version_config_template,
        instance_config=instance_config,
    ) as (plugin, version, instance):
        with upload_file(client, "utterances.tsv") as file:
            assert len(file.refresh().data.blocks) == 0
            file.blockify(plugin_instance=instance.handle).wait()
            # Check the number of blocks
            blocks = file.refresh().data.blocks
            assert len(blocks) == 5
            # Check if the tags are correctly added
            for block in blocks:
                assert block.tags is not None
                assert len(block.tags) > 0
                for tag in block.tags:
                    assert tag.name is not None
                    assert tag.kind is not None
            file.delete()



from abc import ABC
from typing import Any

from steamship.base import Client
from steamship.client.operations.corpus_importer import (
    CorpusImportRequest,
    CorpusImportResponse,
)
from steamship.plugin.service import PluginRequest, PluginService


# Note!
# =====
#
# This is the PLUGIN IMPLEMENTOR's View of a Corpus Importer.
#
# If you are using the Steamship Client, you probably want steamship.client.operations.corpus_importer instead
# of this file.
#
class CorpusImporter(PluginService[CorpusImportRequest, CorpusImportResponse], ABC):
    @classmethod
    def subclass_request_from_dict(
        cls, d: Any, client: Client = None
    ) -> PluginRequest[CorpusImportRequest]:
        return CorpusImportRequest.from_dict(d, client=client)



from .client import Client
from .configuration import Configuration
from .error import SteamshipError
from .metadata import metadata_to_str, str_to_metadata
from .mime_types import MimeTypes
from .request import Request
from .response import Response
from .tasks import Task, TaskComment, TaskState



from steamship.data.plugin import TrainingPlatform
from steamship.plugin.inputs.training_parameter_plugin_input import (
    TrainingParameterPluginInput,
)

__copyright__ = "Steamship"
__license__ = "MIT"

import math

from tests.demo_apps.plugins.taggers.plugin_trainable_tagger import (
    TestTrainableTaggerPlugin,
)

from .. import APPS_PATH
from ..utils.client import get_steamship_client
from ..utils.deployables import deploy_plugin


def test_get_training_parameters():
    """Any trainable plugin needs a Python+Lambda component that can report its training params.
    This tests that all the plumbing works for that to be returned"""
    client = get_steamship_client()
    tagger_path = APPS_PATH / "plugins" / "taggers" / "plugin_trainable_tagger.py"
    # Now make a trainable tagger to train on those tags
    with deploy_plugin(
        client,
        tagger_path,
        "tagger",
        training_platform=TrainingPlatform.managed,
    ) as (tagger, taggerVersion, taggerInstance):
        training_request = TrainingParameterPluginInput(
            pluginInstance=taggerInstance.handle
        )
        res = taggerInstance.get_training_parameters(training_request)
        assert res.data is not None
        params = res.data

        assert params.trainingEpochs is not None
        assert (
            params.trainingEpochs == TestTrainableTaggerPlugin.RESPONSE.trainingEpochs
        )
        assert params.modelName == TestTrainableTaggerPlugin.RESPONSE.modelName
        assert math.isclose(
            params.testingHoldoutPercent,
            TestTrainableTaggerPlugin.RESPONSE.testingHoldoutPercent,
            abs_tol=0.0001,
        )
        assert (
            params.trainingParams == TestTrainableTaggerPlugin.RESPONSE.trainingParams
        )



from steamship import PluginInstance
from steamship.data.embeddings import EmbeddedItem

__copyright__ = "Steamship"
__license__ = "MIT"

from tests.utils.client import get_steamship_client
from tests.utils.random import random_index, random_name


def _list_equal(actual, expected):
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_basic_task_comment():
    steamship = get_steamship_client()
    embedder = PluginInstance.create(steamship, plugin_handle="test-embedder").data
    with random_index(steamship, embedder.handle) as index:
        item1 = EmbeddedItem(
            value="Pizza", externalId="pizza", externalType="food", metadata=[1, 2, 3]
        )

        index.insert(
            item1.value,
            external_id=item1.externalId,
            external_type=item1.externalType,
            metadata=item1.metadata,
        )
        task = index.embed()
        task.wait()

        res2 = index.search(item1.value, include_metadata=True, k=1)
        res2.add_comment(external_id="Foo", external_type="Bar", metadata=[1, 2])
        # We don't return to Res2 until the end to make sure we aren't co-mingling comments!

        res = index.search(item1.value, include_metadata=True, k=1)

        assert res.data.items is not None
        assert len(res.data.items) == 1
        assert res.data.items[0].value.value == item1.value
        assert res.data.items[0].value.external_id == item1.externalId
        assert res.data.items[0].value.external_type == item1.externalType
        _list_equal(res.data.items[0].value.metadata, item1.metadata)

        res.add_comment(external_id="Foo", external_type="Bar", metadata=[1, 2])

        comments = res.list_comments()
        assert len(comments.data.comments) == 1

        comment = comments.data.comments[0]
        assert comment.external_id == "Foo"
        assert comment.external_type == "Bar"
        _list_equal(comment.metadata, [1, 2])

        res.delete_comment(comment)

        comments = res.list_comments()
        assert len(comments.data.comments) == 0

        # Now let's add one
        res.add_comment(external_id="Foo1", external_type="Bar1", metadata=[1, 2, 3])
        res.add_comment(external_id="Foo2", external_type="Bar2", metadata=[1, 2, 3, 4])

        comments = res.list_comments()
        assert len(comments.data.comments) == 2

        comment = comments.data.comments[0]
        assert comment.external_id == "Foo1"
        assert comment.external_type == "Bar1"
        _list_equal(comment.metadata, [1, 2, 3])

        comment = comments.data.comments[1]
        assert comment.external_id == "Foo2"
        assert comment.external_type == "Bar2"
        _list_equal(comment.metadata, [1, 2, 3, 4])

        res.delete_comment(comments.data.comments[0])
        res.delete_comment(comments.data.comments[1])

        comments = res.list_comments()
        assert len(comments.data.comments) == 0

        # Now we handle res2
        comments = res2.list_comments()
        assert len(comments.data.comments) == 1
        comment = comments.data.comments[0]
        assert comment.external_id == "Foo"
        assert comment.external_type == "Bar"
        _list_equal(comment.metadata, [1, 2])
        res.delete_comment(comments.data.comments[0])
        comments = res.list_comments()
        assert len(comments.data.comments) == 0


def test_task_comment_feedback_reporting():
    """
    We want to be able to generate reports like this:

    Select Across Gorup    -- externalGroup
    Inputs Seen: XXX       -- Distinct externalId
    Inputs Suggested: YYY  -- Add to metadata
    Inputs Liked / Disliked / Used -- Add to metadata

    So really we just need to test the group aggregation
    """
    client = get_steamship_client()
    embedder = PluginInstance.create(client, plugin_handle="test-embedder").data
    with random_index(client, plugin_instance=embedder.handle) as index:
        item1 = EmbeddedItem(
            value="Pizza", externalId="pizza", externalType="food", metadata=[1, 2, 3]
        )

        group_name_1 = random_name()
        group_name_2 = random_name()

        index.insert(
            item1.value,
            external_id=item1.externalId,
            external_type=item1.externalType,
            metadata=item1.metadata,
        )
        task = index.embed()
        task.wait()

        res = index.search(item1.value, include_metadata=True, k=1)
        res.add_comment(
            external_id="Foo1",
            external_type="Bar1",
            external_group=group_name_1,
            metadata=[1, 2, 3],
        )
        res.add_comment(
            external_id="Foo2",
            external_type="Bar1",
            external_group=group_name_1,
            metadata=[1, 2, 3],
        )
        res.add_comment(
            external_id="Foo2",
            external_type="Bar1",
            external_group=group_name_2,
            metadata=[1, 2, 3],
        )

        comments = res.list_comments()
        assert len(comments.data.comments) == 3

        g1 = client.tasks.list_comments(external_group=group_name_1)
        assert len(g1.data.comments) == 2

        g2 = client.tasks.list_comments(external_group=group_name_2)
        assert len(g2.data.comments) == 1

        g1 = client.tasks.list_comments(
            task_id=res.task.task_id, external_group=group_name_1
        )
        assert len(g1.data.comments) == 2

        g2 = client.tasks.list_comments(
            task_id=res.task.task_id, external_group=group_name_2
        )
        assert len(g2.data.comments) == 1

        g1 = client.tasks.list_comments(
            task_id=res.task.task_id, external_id="Foo1", external_group=group_name_1
        )
        assert len(g1.data.comments) == 1

        g2 = client.tasks.list_comments(
            task_id=res.task.task_id, external_id="Foo1", external_group=group_name_2
        )
        assert len(g2.data.comments) == 0

        res.delete_comment(comments.data.comments[0])
        res.delete_comment(comments.data.comments[1])
        res.delete_comment(comments.data.comments[2])

        g1 = client.tasks.list_comments(external_group=group_name_1)
        assert len(g1.data.comments) == 0

        g2 = client.tasks.list_comments(external_group=group_name_2)
        assert len(g2.data.comments) == 0



import io
import logging
from dataclasses import dataclass
from typing import Any, List, Union, Optional

from steamship.base import Client, Response, Request
from steamship.base.binary_utils import flexi_create
from steamship.base.request import IdentifierRequest
from steamship.data.block import Block
from steamship.data.tags import Tag


class FileUploadType:
    file = "file"  # The CreateRequest contains a file upload that should be used
    url = "url"  # The CreateRequest contains a url that should be scraped
    value = "value"  # The Create Request contains a `text` field that should be used
    fileImporter = "fileImporter"  # The CreateRequest contains a fileImporter handle that should be used
    blocks = "blocks"  # The CreateRequest contains blocks and tags that should be read in directly


_logger = logging.getLogger(__name__)


@dataclass
class FileClearResponse:
    id: str


@dataclass
class FileQueryRequest(Request):
    tagFilterQuery: str


@dataclass
class File:
    """A file."""

    client: Client = None
    id: str = None
    handle: str = None
    mime_type: str = None
    space_id: str = None
    corpus_id: str = None
    blocks: List[Block] = None
    tags: List[Tag] = None
    filename: str = None

    @dataclass
    class CreateRequest:
        value: str = None
        data: str = None
        url: str = None
        filename: str = None
        type: str = None  # FileUploadType: fileImporter | value | url | data
        mimeType: str = None  # TODO: This should work
        corpusId: str = None
        blocks: List[Block.CreateRequest] = None
        tags: List[Tag.CreateRequest] = None
        pluginInstance: str = None

        @staticmethod
        def from_dict(d: Any, client: Client = None) -> "File.CreateRequest":
            return File.CreateRequest(
                value=d.get("value"),
                data=d.get("data"),
                url=d.get("url"),
                type=d.get("type"),
                mimeType=d.get("mimeType"),
                corpusId=d.get("corpusId"),
                pluginInstance=d.get("pluginInstance"),
                blocks=[
                    Block.CreateRequest.from_dict(block, client=client)
                    for block in d.get("blocks", [])
                ],
                tags=[
                    Tag.CreateRequest.from_dict(tag, client=client)
                    for tag in d.get("tags", [])
                ],
                filename=d.get("filename", None),
            )

        def to_dict(self) -> dict:
            return dict(
                value=self.value,
                data=self.data,
                url=self.url,
                type=self.type,
                mimeType=self.mimeType,
                corpusId=self.corpusId,
                pluginInstance=self.pluginInstance,
                blocks=[block.to_dict() for block in self.blocks]
                if self.blocks
                else [],
                tags=[tag.to_dict() for tag in self.tags] if self.tags else [],
                filename=self.filename,
            )

    @dataclass
    class CreateResponse:
        data: Any = None
        mimeType: str = None

        def __init__(
            self,
            data: Any = None,
            string: str = None,
            bytes: Union[bytes, io.BytesIO] = None,
            json: io.BytesIO = None,
            mime_type: str = None,
        ):
            data, mime_type, encoding = flexi_create(
                data=data, string=string, json=json, bytes=bytes, mime_type=mime_type
            )
            self.data = data
            self.mimeType = mime_type

        # noinspection PyUnusedLocal
        @staticmethod
        def from_dict(d: Any, client: Client = None) -> "File.CreateResponse":
            return File.CreateResponse(
                data=d.get("data", None), mime_type=d.get("mimeType", None)
            )

        def to_dict(self) -> dict:
            return dict(data=self.data, mimeType=self.mimeType)

    @dataclass
    class ListRequest(Request):
        corpusId: str = None

    @dataclass
    class ListResponse:
        files: List["File"]

        @staticmethod
        def from_dict(d: Any, client: Client = None) -> "File.ListResponse":
            return File.ListResponse(
                files=[File.from_dict(f, client=client) for f in d.get("files", [])]
            )

    @dataclass
    class RawRequest(Request):
        id: str

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "Optional[File]":
        # TODO (enias): Resolve code duplication
        if d is None:
            return None
        if "file" in d:
            d = d["file"]
        return File(
            client=client,
            id=d.get("id"),
            handle=d.get("handle"),
            mime_type=d.get("mimeType"),
            corpus_id=d.get("corpusId"),
            space_id=d.get("spaceId"),
            blocks=[
                Block.from_dict(block, client=client) for block in d.get("blocks", [])
            ],
            tags=[Tag.from_dict(tag, client=client) for tag in d.get("tags", [])],
        )

    def to_dict(self) -> dict:
        # TODO (enias): Resolve code duplication
        return dict(
            id=self.id,
            handle=self.handle,
            mimeType=self.mime_type,
            corpusId=self.corpus_id,
            spaceId=self.space_id,
            blocks=[block.to_dict() for block in self.blocks] if self.blocks else [],
            tags=[tag.to_dict() for tag in self.tags] if self.tags else [],
        )

    def delete(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> "Response[File]":
        return self.client.post(
            "file/delete",
            IdentifierRequest(id=self.id),
            expect=File,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def clear(
        self, space_id: str = None, space_handle: str = None, space: Any = None
    ) -> Response[FileClearResponse]:
        return self.client.post(
            "file/clear",
            IdentifierRequest(id=self.id),
            expect=FileClearResponse,
            id_query=self,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    @staticmethod
    def get(
        client: Client,
        id: str = None,
        handle: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response["File"]:  # TODO (Enias): Why is this a staticmethod?
        return client.post(
            "file/get",
            IdentifierRequest(id=id, handle=handle),
            expect=File,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    @staticmethod
    def create(
        client: Client,
        filename: str = None,
        url: str = None,
        content: str = None,
        plugin_instance: str = None,
        mime_type: str = None,
        blocks: List[Block.CreateRequest] = None,
        tags: List[Tag.CreateRequest] = None,
        corpus_id: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> "Response[File]":

        if (
            filename is None
            and content is None
            and url is None
            and plugin_instance is None
            and blocks is None
        ):
            raise Exception(
                "Either filename, content, url, or plugin Instance must be provided."
            )

        if blocks is not None:
            upload_type = FileUploadType.blocks
        elif plugin_instance is not None:
            upload_type = FileUploadType.fileImporter
        elif content is not None:
            # We're still going to use the file upload method for file uploads
            upload_type = FileUploadType.file
        elif filename is not None:
            with open(filename, "rb") as f:
                content = f.read()
            upload_type = FileUploadType.file
        else:
            if url is not None:
                raise Exception(
                    "Unable to determine upload type. For scraping a URL, use the File.scrape method."
                )
            else:
                raise Exception("Unable to determine upload type.")

        req = File.CreateRequest(
            type=upload_type,
            corpusId=corpus_id,
            url=url,
            mimeType=mime_type,
            pluginInstance=plugin_instance,
            blocks=blocks,
            tags=tags,
            filename=filename,
        )

        # Defaulting this here, as opposed to in the Engine, because it is processed by Vapor
        file_part_name = filename if filename else "unnamed"
        return client.post(
            "file/create",
            payload=req,
            file=(file_part_name, content, "multipart/form-data")
            if upload_type != FileUploadType.blocks
            else None,
            expect=File,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    @staticmethod
    def list(
        client: Client,
        corpus_id: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ):
        req = File.ListRequest(corpusId=corpus_id)
        res = client.post(
            "file/list",
            payload=req,
            expect=File.ListResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )
        return res

    @staticmethod
    def scrape(
        client: Client,
        url: str,
        corpus_id: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> "Response[File]":
        req = File.CreateRequest(type=FileUploadType.url, url=url, corpusId=corpus_id)

        return client.post(
            "file/create",
            payload=req,
            expect=File,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )

    def refresh(self):
        return File.get(self.client, self.id)

    @staticmethod
    def query(
        client: Client,
        tag_filter_query: str,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> Response["FileQueryResponse"]:

        req = FileQueryRequest(tagFilterQuery=tag_filter_query)
        res = client.post(
            "file/query",
            payload=req,
            expect=FileQueryResponse,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
        )
        return res

    def raw(self, space_id: str = None, space_handle: str = None, space: Any = None):
        req = File.RawRequest(
            id=self.id,
        )

        return self.client.post(
            "file/raw",
            payload=req,
            space_id=space_id,
            space_handle=space_handle,
            space=space,
            raw_response=True,
        )

    @staticmethod
    def upload(
        client: Client,
        filename: str = None,
        content: str = None,
        mime_type: str = None,
        corpus_id: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ) -> "Response[File]":
        pass

    def blockify(self, plugin_instance: str = None):
        pass

    def tag(
        self,
        plugin_instance: str = None,
        space_id: str = None,
        space_handle: str = None,
        space: Any = None,
    ):
        pass


@dataclass
class FileQueryResponse:
    files: List[File]

    @staticmethod
    def from_dict(d: Any, client: Client = None) -> "FileQueryResponse":
        return FileQueryResponse(
            files=[File.from_dict(file, client=client) for file in d.get("files", [])]
        )
